
#include <cvirte.h>    /* Needed if linking in external compiler; harmless otherwise */
#include <utility.h>
#include <userint.h>
#include <ansi_c.h>
#include "toolbar.h"
#include "tooldemo.h"


/********************/
/*      Macros      */
/********************/

#define kPopupTitle     "Toolbar Test Message"


/********************/
/*      Globals     */
/********************/

static ToolbarType gToolbar;


/********************************/
/*  Private Function Prototypes */
/********************************/

static void SetActiveItems (int menuBar, int menuItem, int startItem,
                            int count, int *active);

static void ShowAction (int parentPanel, char *action);


/************************/
/*      Functions       */
/************************/

static void SetActiveItems (int menuBar, int menuItem, int startItem,
                            int count, int *active)
{
    int i, index;
    char itemName[50];

    Toolbar_GetIndexFromMenu (gToolbar, startItem, &index);
    if (index == -1)
        return;

    *active = !(*active);
    SetMenuBarAttribute (menuBar, menuItem, ATTR_CHECKED, *active);
    for (i = 0 ; i < count ; i++)
        Toolbar_ActivateItem (gToolbar, index + i, *active, 0);
    Toolbar_Display (gToolbar);
}

static void ShowAction (int parentPanel, char *action)
{
    static int actionPanel = 0;
    int top, left, height, width;

    if (!actionPanel) {
        actionPanel = LoadPanel (parentPanel, "tooldemo.uir", ACTION);
        if (actionPanel <= 0) {
            actionPanel = 0;
            return;
        }
    }

    SetCtrlVal (actionPanel, ACTION_ACTION, action);
    GetCtrlBoundingRect (actionPanel, ACTION_ACTION, NULL, NULL, &height,
                         &width);
    SetPanelAttribute (actionPanel, ATTR_HEIGHT, height);
    SetPanelAttribute (actionPanel, ATTR_WIDTH, width);
    SetPanelPos (actionPanel, VAL_AUTO_CENTER, VAL_AUTO_CENTER);
    ResetTimer (actionPanel, ACTION_TIMER);
    SetCtrlAttribute (actionPanel, ACTION_TIMER, ATTR_ENABLED, TRUE);
    DisplayPanel (actionPanel);
}

int CVICALLBACK ActionTimerCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_TIMER_TICK:
            HidePanel (panel);
            SetCtrlAttribute (panel, control, ATTR_ENABLED, FALSE);
            break;
    }
    return 0;
}

int CVICALLBACK ActionPanelCallback (int panel, int event, void *callbackData,
        int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_PANEL_SIZE:
        case EVENT_PANEL_MOVE:
        case EVENT_GOT_FOCUS:
            break;
        default:
            SetCtrlAttribute (panel, ACTION_TIMER, ATTR_ENABLED, FALSE);
            HidePanel (panel);
            break;
    }

    return (FALSE);
}

void CVICALLBACK ViewOtherItems (int menuBar, int menuItem, void *callbackData,
        int panel)
{
    static int isActive = TRUE;

    SetActiveItems (menuBar, menuItem, MAINMENU_HELP, 4, &isActive);
}

void CVICALLBACK ViewSortItems (int menuBar, int menuItem, void *callbackData,
        int panel)
{
    static int isActive = TRUE;

    SetActiveItems (menuBar, menuItem, MAINMENU_SORT_BYNAME, 5, &isActive);
}

void CVICALLBACK ViewFindItems (int menuBar, int menuItem, void *callbackData,
        int panel)
{
    static int isActive = TRUE;

    SetActiveItems (menuBar, menuItem, MAINMENU_EDIT_FIND, 4, &isActive);
}

void CVICALLBACK ViewEditItems (int menuBar, int menuItem, void *callbackData,
        int panel)
{
    static int isActive = TRUE;

    SetActiveItems (menuBar, menuItem, MAINMENU_EDIT_CUT, 6, &isActive);
}

void CVICALLBACK ViewFileItems (int menuBar, int menuItem, void *callbackData,
        int panel)
{
    static int isActive = TRUE;

    SetActiveItems (menuBar, menuItem, MAINMENU_FILE_NEW, 5, &isActive);
}

void CVICALLBACK Help (int menuBar, int menuItem, void *callbackData,
        int panel)
{
    MessagePopup ("Toolbar Demo",
                  
                  "This toolbar works just like the toolbars in LabWindows/CVI,\n"
                  "with the exception that the panel/window does not have to\n"
                  "be active for the toolbar help to function.  For example,\n"
                  "if you move this popup so that it doesn't obscure the\n"
                  "toolbar, you will see that you can still use the toolbar\n"
                  "help even though clicking on the toolbar has no effect.\n\n"

                  "All of the toolbar items in this example correspond to\n"
                  "items in the panel menubar.\n\n"

                  "Try the items in the View menu for a simple example of\n"
                  "how you might make your toolbar customizable.");
}

void CVICALLBACK WindowTile (int menuBar, int menuItem, void *callbackData,
        int panel)
{
    ShowAction (panel, "Window Tile");
}

void CVICALLBACK WindowCascade (int menuBar, int menuItem, void *callbackData,
        int panel)
{
    ShowAction (panel, "Window Cascade");
}

void CVICALLBACK SortByName (int menuBar, int menuItem, void *callbackData,
        int panel)
{
    int index;
    
    Toolbar_GetIndexFromMenu (gToolbar, MAINMENU_SORT_BYNAME, &index);
    Toolbar_SetItemVal (gToolbar, index, 1);
    
    ShowAction (panel, "Sort by Name");
}

void CVICALLBACK SortByType (int menuBar, int menuItem, void *callbackData,
        int panel)
{
    int index;
    
    Toolbar_GetIndexFromMenu (gToolbar, MAINMENU_SORT_BYTYPE, &index);
    Toolbar_SetItemVal (gToolbar, index, 1);
    
    ShowAction (panel, "Sort by Type");
}

void CVICALLBACK SortBySize (int menuBar, int menuItem, void *callbackData,
        int panel)
{
    int index;
    
    Toolbar_GetIndexFromMenu (gToolbar, MAINMENU_SORT_BYSIZE, &index);
    Toolbar_SetItemVal (gToolbar, index, 1);
    
    ShowAction (panel, "Sort by Size");
}

void CVICALLBACK SortByDate (int menuBar, int menuItem, void *callbackData,
        int panel)
{
    int index;
    
    Toolbar_GetIndexFromMenu (gToolbar, MAINMENU_SORT_BYDATE, &index);
    Toolbar_SetItemVal (gToolbar, index, 1);
    
    ShowAction (panel, "Sort by Date");
}

void CVICALLBACK ViewWatch (int menuBar, int menuItem, void *callbackData,
        int panel)
{
    ShowAction (panel, "View Watch");
}

void CVICALLBACK EditReplace (int menuBar, int menuItem, void *callbackData,
        int panel)
{
    ShowAction (panel, "Edit Replace");
}

void CVICALLBACK EditFind (int menuBar, int menuItem, void *callbackData,
        int panel)
{
    ShowAction (panel, "Edit Find");
}


void CVICALLBACK ViewToolbar (int menuBar, int menuItem, void *callbackData,
        int panel)
{
    static int visible = TRUE;

    visible = !visible;
    SetMenuBarAttribute (menuBar, menuItem, ATTR_CHECKED, visible);

    if (visible)
        Toolbar_Display (gToolbar);
    else
        Toolbar_Hide (gToolbar);
}

void CVICALLBACK EditRedo (int menuBar, int menuItem, void *callbackData,
        int panel)
{
    ShowAction (panel, "Edit Redo");
}

void CVICALLBACK EditUndo (int menuBar, int menuItem, void *callbackData,
        int panel)
{
    ShowAction (panel, "Edit Undo");
}

void CVICALLBACK EditPaste (int menuBar, int menuItem, void *callbackData,
        int panel)
{
    ShowAction (panel, "Edit Paste");
}

void CVICALLBACK EditCopy (int menuBar, int menuItem, void *callbackData,
        int panel)
{
    ShowAction (panel, "Edit Copy");
}

void CVICALLBACK EditCut (int menuBar, int menuItem, void *callbackData,
        int panel)
{
    ShowAction (panel, "Edit Cut");
}

void CVICALLBACK FileQuit (int menuBar, int menuItem, void *callbackData,
        int panel)
{
    QuitUserInterface (0);
}

void CVICALLBACK FileSave (int menuBar, int menuItem, void *callbackData,
        int panel)
{
    ShowAction (panel, "File Save");
}

int CVICALLBACK MainPanelCallback (int panel, int event, void *callbackData,
        int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_CLOSE:
            QuitUserInterface (0);
            break;
    }
    return 0;
}

void CVICALLBACK FileOpen (int menuBar, int menuItem, void *callbackData,
        int panel)
{
    ShowAction (panel, "File Open");
}

void CVICALLBACK FileNew (int menuBar, int menuItem, void *callbackData,
        int panel)
{
    ShowAction (panel, "File New");
}


void CVICALLBACK FilePrint (int menuBar, int menuItem, void *callbackData,
        int panel)
{
    ShowAction (panel, "File Print");
}


void CVICALLBACK MenuDimmerCallback (int menuBar, int panel)
{
    int toolbarOn;

    GetMenuBarAttribute (menuBar, MAINMENU_VIEW_TOOLBAR, ATTR_CHECKED,
                         &toolbarOn);

    SetMenuBarAttribute (menuBar, MAINMENU_VIEW_FILEITEMS, ATTR_DIMMED,
                         !toolbarOn);
    SetMenuBarAttribute (menuBar, MAINMENU_VIEW_EDITITEMS, ATTR_DIMMED,
                         !toolbarOn);
    SetMenuBarAttribute (menuBar, MAINMENU_VIEW_FINDITEMS, ATTR_DIMMED,
                         !toolbarOn);
    SetMenuBarAttribute (menuBar, MAINMENU_VIEW_SORTITEMS, ATTR_DIMMED,
                         !toolbarOn);
    SetMenuBarAttribute (menuBar, MAINMENU_VIEW_OTHERITEMS, ATTR_DIMMED,
                         !toolbarOn);
}


int main (int argc, char *argv[])
{
    int panel;

    if (InitCVIRTE (0, argv, 0) == 0)    /* Needed if linking in external compiler; harmless otherwise */
        return -1;    /* out of memory */

    panel = LoadPanel (0, "tooldemo.uir", PANEL);
    InstallMenuDimmerCallback (GetPanelMenuBar (panel),
                               MenuDimmerCallback);
    if (Toolbar_New (panel, GetPanelMenuBar(panel), "", 0, 0, 1, 0, &gToolbar) < 0)
    {
        MessagePopup ("Toolbar Demo", "Failed to create toolbar!");
        exit (-1);
    }

    Toolbar_InsertItem (gToolbar, END_OF_LIST, kCommandButton, 1, "New",
                        kMenuCallback, MAINMENU_FILE_NEW, NULL, NULL,
                        "buttons/new.pcx");
    Toolbar_InsertItem (gToolbar, END_OF_LIST, kCommandButton, 1, "Open",
                        kMenuCallback, MAINMENU_FILE_OPEN, NULL, NULL,
                        "buttons/open.pcx");
    Toolbar_InsertItem (gToolbar, END_OF_LIST, kCommandButton, 1, "Save",
                        kMenuCallback, MAINMENU_FILE_SAVE, NULL, NULL,
                        "buttons/save.pcx");
    Toolbar_InsertItem (gToolbar, END_OF_LIST, kCommandButton, 1, "Print",
                        kMenuCallback, MAINMENU_FILE_PRINT, NULL, NULL,
                        "buttons/print.pcx");
    Toolbar_InsertItem (gToolbar, END_OF_LIST, kSeparator, 1, NULL,
                        kNoCallback, 0, NULL, NULL, NULL);

    Toolbar_InsertItem (gToolbar, END_OF_LIST, kCommandButton, 1, "Cut",
                        kMenuCallback, MAINMENU_EDIT_CUT, NULL, NULL,
                        "buttons/cut.pcx");
    Toolbar_InsertItem (gToolbar, END_OF_LIST, kCommandButton, 1, "Copy",
                        kMenuCallback, MAINMENU_EDIT_COPY, NULL, NULL,
                        "buttons/copy.pcx");
    Toolbar_InsertItem (gToolbar, END_OF_LIST, kCommandButton, 1,
                        "Paste", kMenuCallback, MAINMENU_EDIT_PASTE, NULL,
                        NULL, "buttons/paste.pcx");
    Toolbar_InsertItem (gToolbar, END_OF_LIST, kCommandButton, 1, "Undo",
                        kMenuCallback, MAINMENU_EDIT_UNDO, NULL, NULL,
                        "buttons/undo.pcx");
    Toolbar_InsertItem (gToolbar, END_OF_LIST, kCommandButton, 1, "Redo",
                        kMenuCallback, MAINMENU_EDIT_REDO, NULL, NULL,
                        "buttons/redo.pcx");
    Toolbar_InsertItem (gToolbar, END_OF_LIST, kSeparator, 1, NULL,
                        kNoCallback, 0, NULL, NULL, NULL);

    Toolbar_InsertItem (gToolbar, END_OF_LIST, kCommandButton, 1, "Find",
                        kMenuCallback, MAINMENU_EDIT_FIND, NULL, NULL,
                        "buttons/find.pcx");
    Toolbar_InsertItem (gToolbar, END_OF_LIST, kCommandButton, 1,
                        "Replace", kMenuCallback, MAINMENU_EDIT_REPLACE,
                        NULL, NULL, "buttons/replace.pcx");
    Toolbar_InsertItem (gToolbar, END_OF_LIST, kCommandButton, 1, "Watch",
                        kMenuCallback, MAINMENU_VIEW_WATCH, NULL, NULL,
                        "buttons/glasses.pcx");
    Toolbar_InsertItem (gToolbar, END_OF_LIST, kSeparator, 1, NULL,
                        kNoCallback, 0, NULL, NULL, NULL);

    Toolbar_InsertItem (gToolbar, END_OF_LIST, kExclusiveToggleButton, 1,
                        "Sort by Name", kMenuCallback, MAINMENU_SORT_BYNAME, NULL,
                        NULL, "buttons/byname.pcx");
    Toolbar_InsertItem (gToolbar, END_OF_LIST, kExclusiveToggleButton, 1,
                        "Sort by Type", kMenuCallback, MAINMENU_SORT_BYTYPE, NULL,
                        NULL, "buttons/bytype.pcx");
    Toolbar_InsertItem (gToolbar, END_OF_LIST, kExclusiveToggleButton, 1,
                        "Sort by Size", kMenuCallback, MAINMENU_SORT_BYSIZE, NULL,
                        NULL, "buttons/bysize.pcx");
    Toolbar_InsertItem (gToolbar, END_OF_LIST, kExclusiveToggleButton, 1,
                        "Sort by Date", kMenuCallback, MAINMENU_SORT_BYDATE, NULL,
                        NULL, "buttons/bydate.pcx");
    Toolbar_InsertItem (gToolbar, END_OF_LIST, kSeparator, 1, NULL,
                        kNoCallback, 0, NULL, NULL, NULL);

    Toolbar_InsertItem (gToolbar, END_OF_LIST, kCommandButton, 1, "Help",
                        kMenuCallback, MAINMENU_HELP, NULL, NULL,
                        "buttons/help.pcx");
    Toolbar_InsertItem (gToolbar, END_OF_LIST, kCommandButton, 1,
                        "Cascade", kMenuCallback, MAINMENU_WINDOW_CASCADE,
                        NULL, NULL, "buttons/cascade.pcx");
    Toolbar_InsertItem (gToolbar, END_OF_LIST, kCommandButton, 1, "Tile",
                        kMenuCallback, MAINMENU_WINDOW_TILE, NULL, NULL,
                        "buttons/tile.pcx");
    Toolbar_InsertItem (gToolbar, END_OF_LIST, kSeparator, 1, NULL,
                        kNoCallback, 0, NULL, NULL, NULL);

    Toolbar_InsertItem (gToolbar, END_OF_LIST, kCommandButton, 1, "Quit",
                        kMenuCallback, MAINMENU_FILE_QUIT, NULL, NULL,
                        "buttons/stop.pcx");

    Toolbar_Display (gToolbar);
    DisplayPanel (panel);
    RunUserInterface();
    
    return 0;
}

