
#include <cvirte.h>    /* Needed if linking in external compiler; harmless otherwise */
#include <userint.h>
#include <utility.h>
#include <string.h>
#include "sndplay.h"
#include "playsnd.h"


#define TRUE    1
#define FALSE   0

typedef struct {
    int     async;
    int     loop;
    int     playingAll;
    int     stopPressed;
    } SoundStatusType;
    
int thePanel;

char *defaultDirs[] = {"c:\\windows", "c:\\chicago\\media", "c:\\win95\\media"};


SoundStatusType soundStatus;

int GetCurrentlySelectedFile(int panel, int listbox, char *filename);
int GetPlayOptions(int panel, int *asynchronous, int *loop);
void PlaySingleSound(int panel, int listbox);
void PlayAllSounds(int panel, int listbox);
int DirectoryExists(char *directory);
void GetInitialDirectory(char *directory);



int main (int argc, char *argv[])
{
    char initialDirectory[MAX_PATHNAME_LEN];

    if (InitCVIRTE (0, argv, 0) == 0)    /* Needed if linking in external compiler; harmless otherwise */
        return -1;    /* out of memory */
    
    thePanel = LoadPanel (0, "sndplay.uir", PANEL);

    GetInitialDirectory(initialDirectory);
    SetCtrlVal(thePanel, PANEL_DIRECTORY, initialDirectory);
    FillListBoxWithWavsInDir(thePanel, PANEL_LISTBOX, initialDirectory);

    DisplayPanel (thePanel);
    RunUserInterface ();
    
    return 0;
}

int CVICALLBACK Quit (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            QuitUserInterface (0);
            break;
    }
    return 0;
}

int CVICALLBACK SoundListCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            PlaySingleSound(panel, PANEL_LISTBOX);
            break;
    }
    return 0;
}

int CVICALLBACK PlayAll (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            PlayAllSounds(panel, PANEL_LISTBOX);
            break;
    }
    return 0;
}

int CVICALLBACK PlayOne (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            PlaySingleSound(panel, PANEL_LISTBOX);
            break;
    }
    return 0;
}

int GetCurrentlySelectedFile(int panel, int listbox, char *filename)
{
    int index;
    int error;
    
    GetCtrlIndex (panel, PANEL_LISTBOX, &index);
    
    GetValueFromIndex (panel, PANEL_LISTBOX, index, filename);
    
    return 0;   

}

int GetPlayOptions(int panel, int *asynchronous, int *loop)
{
    GetCtrlVal (panel, PANEL_LOOP, loop);

    SetCtrlAttribute (panel, PANEL_ASYNCHRONOUS, ATTR_DIMMED, *loop);
    
    if (loop) 
        *asynchronous = 1;
    else
        GetCtrlVal (panel, PANEL_ASYNCHRONOUS, asynchronous);

    return 0;
}

int CVICALLBACK ChangeLoopMode (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    int async, loop;
    
    switch (event)
    {
        case EVENT_COMMIT:
            GetPlayOptions (panel, &async, &loop);
            break;
    }
    return 0;
}

void PlaySingleSound(int panel, int listbox)
{
    char filename[MAX_PATHNAME_LEN];
    
    GetCurrentlySelectedFile(panel, listbox, filename);
    
    GetPlayOptions(panel, &soundStatus.async, &soundStatus.loop);
    
    PlaySoundEx (filename, soundStatus.async, soundStatus.loop, 0, 1);

}

void PlayAllSounds(int panel, int listbox)
{
    int index;
    int count;
    char filename[MAX_PATHNAME_LEN];
    
    GetNumListItems (panel, listbox, &count);

    soundStatus.playingAll = TRUE;
    soundStatus.stopPressed = FALSE;
    
    for (index = 0; index < count; index++) {
        ProcessSystemEvents();
        if (soundStatus.stopPressed)
            break;
        GetValueFromIndex (panel, listbox, index, filename);
        SetCtrlIndex (panel, listbox, index);
        ProcessSystemEvents();
        PlaySoundEx (filename, 0, 0, 0, 1);
    }
    
    soundStatus.playingAll = FALSE;
    
}

int CVICALLBACK Stop (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            if (soundStatus.playingAll)
                soundStatus.stopPressed = TRUE;
            else if (soundStatus.async || soundStatus.loop) {
                StopCurrentSound ();
                soundStatus.async = soundStatus.loop = FALSE;
            }
            break;
    }
    return 0;
}

int CVICALLBACK ChangeDir (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    char currentDirectory[MAX_PATHNAME_LEN];
    char newDirectory[MAX_PATHNAME_LEN];
    
    switch (event) {
        case EVENT_COMMIT:
            GetCtrlVal (panel, PANEL_DIRECTORY, currentDirectory);
            if (DirSelectPopup (currentDirectory, "Select Directory", 1, 0,
                                newDirectory) != VAL_NO_DIRECTORY_SELECTED) {
                if (strcmp(currentDirectory, newDirectory)) {
                    SetCtrlVal(panel, PANEL_DIRECTORY, newDirectory);
                    FillListBoxWithWavsInDir(panel, PANEL_LISTBOX, newDirectory);
                }
            }
            break;
    }
    return 0;
}

int FillListBoxWithWavsInDir(int panel, int listbox, char *directory)
{
    char fileSpec[MAX_PATHNAME_LEN];
    char fileName[MAX_FILENAME_LEN];
    char fullPath[MAX_PATHNAME_LEN];
    
    ClearListCtrl (panel, listbox);
    
    MakePathname (directory, "*.wav", fileSpec);
    if (GetFirstFile (fileSpec, 1, 0, 0, 0, 0, 0, fileName) == 0) {
        do {
            MakePathname (directory, fileName, fullPath);
            InsertListItem (panel, listbox, -1, fileName, fullPath);
        } while (GetNextFile (fileName) == 0);
    }
    
    return 0;
}


void GetInitialDirectory(char *directory)
{
    int dummy, i, breakOnLibErrState;
    
    breakOnLibErrState = GetBreakOnLibraryErrors ();
    DisableBreakOnLibraryErrors ();
    
    for (i = 0; i < sizeof(defaultDirs) / sizeof(defaultDirs[0]); i++)
        if (DirectoryExists(defaultDirs[i])) {
            strcpy(directory, defaultDirs[i]);
            goto done;
        }
    DirSelectPopup ("c:\\", "Select Waves Directory", 0, 0, directory);
done:
    if (breakOnLibErrState)
        EnableBreakOnLibraryErrors ();
    else
        DisableBreakOnLibraryErrors ();
        
                                                                           
}


int DirectoryExists(char *directory)
{   
    int dummy;
    
    if (GetFileAttrs (directory, &dummy, &dummy, &dummy, &dummy) == 1)
        return TRUE;
    else
        return FALSE;
}
