/*
    This is a skeleton program to demonstrate how you would write
    a TCP Client program. This program allows a client to connect
    to a server, and communicates via a user interface panel.

    Assumption: Only connecting to 1 server.
*/

#include <cvirte.h>    /* Needed if linking in external compiler; harmless otherwise */
#include <stdio.h>
#include <stdlib.h>
#include <tcpsupp.h>
#include <string.h>
#include <utility.h>
#include <userint.h>

#include "tcpserv.h"

int CVICALLBACK ClientCallback(unsigned handle, int event, int error, void *callbackData);

int         panelId, quit=0;
unsigned    gHandle;

int main (int argc, char *argv[])
{
    char        serverName[256];
    char        portNumString[32];
    char        buffer[512];
    int         portNumber;
    int         h, ctrl;
    unsigned    handle;

    if (InitCVIRTE (0, argv, 0) == 0)    /* Needed if linking in external compiler; harmless otherwise */
        return -1;    /* out of memory */
    
    DisableBreakOnLibraryErrors();

    /*--------------------------------------*/
    /* prompt for server name to connect to */
    /*--------------------------------------*/
    PromptPopup ("Server Name?",
                 "Type the name of the machine running the sample server project\n(example: abc.xyz.com)",
                 serverName, 255);

    /*------------------------------------------------*/
    /* prompt for port number on server to connect to */
    /*------------------------------------------------*/
    PromptPopup ("Port Number?",
                 "Type the port number that was set when starting the sample server project\n(example: 6000)",
                 portNumString, 31);
    portNumber = atoi(portNumString);


    /*---------------------------*/
    /* load user interface panel */
    /*---------------------------*/
    panelId = LoadPanel(0, "tcpserv.uir", PANEL);
    SetPanelAttribute (panelId, ATTR_TITLE, "TCP Client");
    DisplayPanel(panelId);

    /*----------------------------------------------------------*/
    /* attempt to connect to server and wait in user event loop */
    /*----------------------------------------------------------*/
    SetWaitCursor (1);
    if (ConnectToTCPServer (&handle, portNumber, serverName, ClientCallback,
                            NULL, 5000)<0) {
        MessagePopup("TCP Client", "Connection to server failed !");
    } else {
        SetWaitCursor (0);
        SetCtrlVal(panelId, PANEL_CONNECTED, 1);

        SetActiveCtrl(panelId, PANEL_STRING);
        while(!quit){
            if (GetUserEvent(0, &h, &ctrl) == 1) {
                switch (ctrl) {
                    case PANEL_QUIT:
                        quit = 1;
                        break;
                    case PANEL_CLEAR:
                        ResetTextBox(panelId, PANEL_kReceive, "");
                        break;
                    default:
                        break;
                }
            }
        }
        DisconnectFromTCPServer(gHandle);
        DiscardPanel(panelId);
    }

    return 0;
}



/* callback for the command button event */
int CVICALLBACK TxScrCallback(int panelHandle, int controlID, int event, void *callbackData,
            int eventData1, int eventData2)
{
    int     numLines;
    char    buffer[512];

    switch(event) {
        case EVENT_VAL_CHANGED:
            SetCtrlAttribute (panelHandle, controlID, ATTR_CALLBACK_DATA, (void *)1);
            break;
        case EVENT_KEYPRESS:
            if (callbackData == (void *)0) {
                if (eventData1 == VAL_ENTER_VKEY) {
                    GetCtrlVal (panelId, PANEL_STRING, buffer);
                    if (buffer[0] == 0)
                        TxScrCallback (panelHandle, controlID, EVENT_COMMIT, callbackData, 0, 0);
                }
            }
            break;
        case EVENT_COMMIT:
            SetCtrlAttribute (panelHandle, controlID, ATTR_CALLBACK_DATA, (void *)0);
            GetCtrlVal (panelId, PANEL_STRING, buffer);
            strcat (buffer, "\n");
            SetCtrlVal (panelId, PANEL_kTransmit, buffer);
            SetCtrlVal (panelId, PANEL_STRING, "");
            if(ClientTCPWrite (gHandle, buffer, strlen(buffer)+1, 1000)<0) {
                MessagePopup ("TCP Server", "TCP write error!");
                SetCtrlVal(panelId, PANEL_CONNECTED, 0);
                quit = 1;
            }
        default:
            break;
    }
    return 0;
}



/*
    callback function for the client

    it prompts user when server request for close connection.
*/
int CVICALLBACK ClientCallback(unsigned handle, int event, int error, void *callbackData) {
    char        buffer[256];
    int         dataSize=256, i;

    gHandle = handle;
    switch (event) {
        case TCP_DATAREADY:
            if((dataSize = ClientTCPRead(gHandle, buffer, dataSize, 1000))<0) {
                MessagePopup("TCP Client", "TCP read error!");
                SetCtrlVal(panelId, PANEL_CONNECTED, 0);
                quit = 1;
                break;
            }
            SetCtrlVal(panelId, PANEL_kReceive, buffer);
            break;
        case TCP_DISCONNECT:
            MessagePopup("TCP Client", "Server has closed connection!");
            SetCtrlVal(panelId, PANEL_CONNECTED, 0);
            quit = 1;
            break;
    }
    return 0;
}
