/*
    This is a skeleton program to demonstrate how you would write
    a TCP Server program. This program allows a client to connect
    to this server, and communicates via a user interface panel.

    Assumption: Only accepting 1 connection.
*/

#include <cvirte.h>    /* Needed if linking in external compiler; harmless otherwise */
#include <stdio.h>
#include <stdlib.h>
#include <tcpsupp.h>
#include <string.h>
#include <utility.h>
#include <userint.h>
#include "tcpserv.h"


int CVICALLBACK ServerCallback(unsigned handle, int event, int error, void *callbackData);


int         quit=0;
int         panelId;
unsigned    gHandle;

int main (int argc, char *argv[])
{
    char    portNumString[32];
    char    buffer[512];
    int     numLines, ctrl, handle, i;
    int     portNumber;

    if (InitCVIRTE (0, argv, 0) == 0)    /* Needed if linking in external compiler; harmless otherwise */
        return -1;    /* out of memory */
    
    DisableBreakOnLibraryErrors();

    /*-----------------------------------------------*/
    /* prompt for port number to wait for connection */
    /*-----------------------------------------------*/
    PromptPopup ("Port Number?",
                 "Type the port number for client to connect to\n(example: 6000)",
                 portNumString, 31);
    portNumber = atoi(portNumString);

    /*---------------------------*/
    /* load user interface panel */
    /*---------------------------*/
    panelId = LoadPanel(0, "tcpserv.uir", PANEL);
    SetPanelAttribute (panelId, ATTR_CLOSE_CTRL, PANEL_QUIT);
    SetPanelAttribute (panelId, ATTR_TITLE, "TCP Server");
    SetPanelPos (panelId, VAL_AUTO_CENTER, VAL_AUTO_CENTER);
    InstallCtrlCallback (panelId, PANEL_kTransmit, TxScrCallback, 0);
    DisplayPanel(panelId);

    /*---------------------------------------------*/
    /* register server and wait in user event loop */
    /*---------------------------------------------*/
    if (RegisterTCPServer(portNumber, ServerCallback, NULL)<0) {
        MessagePopup ("TCP Server", "RegisterTCPServer failed !!!");
        return 0;
    }
    SetActiveCtrl(panelId, PANEL_STRING);
    while(!quit){
        if (GetUserEvent(0, &handle, &ctrl) == 1) {
            switch (ctrl) {
                case PANEL_QUIT:
                    quit = 1;
                    break;
                case PANEL_CLEAR:
                    ResetTextBox(panelId, PANEL_kReceive, "");
                    break;
                default:
                    break;
            }
        }
    }
    UnregisterTCPServer(portNumber);
    DiscardPanel(panelId);

    return 0;
}


/* callback for the text entry string control */
int CVICALLBACK TxScrCallback(int panelHandle, int controlID, int event, void *callbackData,
            int eventData1, int eventData2)
{
    int     numLines, connected;
    char    buffer[512];

    switch(event) {
        case EVENT_VAL_CHANGED:
            SetCtrlAttribute (panelHandle, controlID, ATTR_CALLBACK_DATA, (void *)1);
            break;
        case EVENT_KEYPRESS:
            if (callbackData == (void *)0) {
                if (eventData1 == VAL_ENTER_VKEY) {
                    GetCtrlVal (panelId, PANEL_STRING, buffer);
                    if (buffer[0] == 0)
                        TxScrCallback (panelHandle, controlID, EVENT_COMMIT, callbackData, 0, 0);
                }
            }
            break;
        case EVENT_COMMIT:
            GetCtrlVal (panelId, PANEL_CONNECTED, &connected);
            if (!connected) {
                MessagePopup ("TCP Server", "Not connected!");
                break;
            }

            GetCtrlVal (panelId, PANEL_STRING, buffer);
            strcat (buffer, "\n");
            SetCtrlVal (panelId, PANEL_kTransmit, buffer);
            SetCtrlVal (panelId, PANEL_STRING, "");
            if(ServerTCPWrite(gHandle, buffer, strlen(buffer)+1, 1000)<0) {
                MessagePopup ("TCP Server", "TCP write error!");
                SetCtrlVal(panelId, PANEL_CONNECTED, 0);
            }
        default:
            break;
    }
    return 0;
}


/*
    callback function for the server

    it prompts user when there is a new connect established, or client request
    for close connection.
*/
int CVICALLBACK ServerCallback(unsigned handle, int event, int error, void *callbackData)
{
    char    buf[512];
    int     readSize;

    switch (event) {
        case TCP_CONNECT:
            gHandle = handle;
            SetCtrlVal(panelId, PANEL_CONNECTED, 1);
            SetCtrlVal(panelId, PANEL_kReceive, "--- New connection established ---\n");
            break;
        case TCP_DATAREADY:
            if((readSize = ServerTCPRead(gHandle, buf, 512, 1000))==-kTCP_ConnectionClosed) {
                MessagePopup("TCP Server", "TCP read error!");
                SetCtrlVal(panelId, PANEL_CONNECTED, 0);
                break;
            }
            SetCtrlVal(panelId, PANEL_kReceive, buf);
            break;
        case TCP_DISCONNECT:
            SetCtrlVal (panelId, PANEL_kReceive, "--- Connection closed by client ---\n");
            SetCtrlVal(panelId, PANEL_CONNECTED, 0);
            break;
    }
    return 0;
}

