/*  Sample program to illustrate how to auto scale the Y-axis of a strip
    chart.
*/

#include <cvirte.h>    /* Needed if linking in external compiler; harmless otherwise */
#include <utility.h>
#include <analysis.h>
#include <userint.h>
#include <ansi_c.h>
#include "toolbox.h"
#include "autostrp.h"

#define MAXTRACES   64
#define NUMCHARTS   3

typedef struct linked_list {
    double              min;
    double              max;
    int                 panel;
    int                 control;
    int                 XRANGE;
    int                 count;
    double              *data;
    double              *datahistory;
    struct linked_list  *nextplot;
    unsigned short      plotid;
} PLOT, *PLOTLINK;

static double           data[MAXTRACES];
static unsigned short   numtraces[NUMCHARTS];
static int              plot[NUMCHARTS] = {FALSE, FALSE, FALSE};
static int              controlIds[NUMCHARTS] = {PANEL_CONTROL, PANEL_CONTROL_2,
                            PANEL_CONTROL_3};  
static int              traceIds[NUMCHARTS] = {PANEL_NUMTRACES, PANEL_NUMTRACES_2,
                            PANEL_NUMTRACES_3};
static int              chartIds[NUMCHARTS] = {PANEL_CHART, PANEL_CHART_2,
                            PANEL_CHART_3};  

static int  PlotAutoStrip (int panel,  int control, double *yArray, int traces);
static void FreeStripBuffer (void);

int main (int argc, char *argv[])
{
    int i, panel;
    
    if (InitCVIRTE (0, argv, 0) == 0)    /* Needed if linking in external compiler; harmless otherwise */
        return -1;    /* out of memory */
    
    srand (Timer());
    
    panel = LoadPanel (0, "autostrp.uir", PANEL);
    for (i = 0; i < NUMCHARTS; i++)
        GetCtrlVal (panel, traceIds[i], &numtraces[i]);
    DisplayPanel (panel);
    RunUserInterface();
    
    FreeStripBuffer();

    return 0;
}

int CVICALLBACK TimerCallback (int panel, int control, int event, void *callbackData, int eventData1,
    int eventData2)
{
    static int  count = 0;
    int         i, j;
    
    if (event != EVENT_TIMER_TICK)
        return FALSE;
    
    count++;
    for (i = 0; i < NUMCHARTS; i++) {
        if (!plot[i])
            continue;
        for (j = 0; j < numtraces[i]; j++)
            data[j] = (double)(3 << j) / (1 << i) * rand() / RAND_MAX *
                sin (count * PI / 100.0);
        PlotAutoStrip (panel, chartIds[i], data, numtraces[i]);
        }
        
    return FALSE;
}


int CVICALLBACK QuitCallback (int panel, int control, int event, void *callbackData, int eventData1,
    int eventData2)
{
    short stat;

    if (event != EVENT_COMMIT)
        return FALSE;
        
    QuitUserInterface (0);
    
    return FALSE;
}


int CVICALLBACK ControlCallback (int panel, int control, int event, void *callbackData, int eventData1,
    int eventData2)
{
    int i;
    
    if (event != EVENT_COMMIT)
        return FALSE;
        
    for (i = 0; i < NUMCHARTS; i++)
        if (control  == controlIds[i])
            break;
        
    plot[i] = !plot[i];
    
    return FALSE;
}


int CVICALLBACK TraceCallback (int panel, int control, int event, void *callbackData, int eventData1,
    int eventData2)
{
    int i;
    
    if (event != EVENT_COMMIT)
        return FALSE;
        
    for (i = 0; i < NUMCHARTS; i++)
        if (control  == traceIds[i])
            break;
            
    FreeStripBuffer();  
    GetCtrlVal (panel, control, &numtraces[i]);
    SetCtrlAttribute (panel, chartIds[i], ATTR_NUM_TRACES, numtraces[i]);
    
    return FALSE;
}

static PLOTLINK         headoflist;
static unsigned short   actualnumplots = 0;
static short            firstcall = TRUE;

static int PlotAutoStrip (int panel,  int control, double *yArray, int traces)
{
    static PLOTLINK         active, tempplotptr;
    static PLOT             tempplot;
    static double           ymin, ymax, xmax, xmin;
    static int              temp1, temp2, start = 0, i;
    static short            match = 0;
    static unsigned short   numplots = 0, currentplot = 0;

    if (!firstcall)
        active = headoflist;

    if (firstcall) {
        firstcall = FALSE;
        numplots = 1;
        actualnumplots = 1;
        active = malloc (sizeof (PLOT));
        tempplotptr = active;
        headoflist = active;
        active->control = control;
        active->plotid = 1;
        active->panel = panel;
        active->count = -1;
        active->nextplot = 0;
        GetAxisRange (panel, control, &temp1, &xmin, &xmax, &temp2, &ymin, &ymax);
        active->XRANGE = (int)(xmax - xmin) + 1;
        active->datahistory = malloc (sizeof (double) * active->XRANGE * traces);
        active->data = malloc (sizeof (double) * traces);
        active->min = 10000;
        active->max = -10000;
        }

    currentplot = active->plotid - 1;
    while (!match) {
        currentplot++;
        if (active->panel != panel || active->control != control)
            if (!active->nextplot) {
                currentplot++;
                numplots++;
                actualnumplots++;
                match = 1;
                active = malloc (sizeof (PLOT));
                tempplotptr->nextplot = active;
                tempplotptr = active;
                active->control = control;
                active->panel = panel;
                active->count = 0;
                active->plotid = currentplot;
                active->nextplot = 0;
                GetAxisRange (panel, control, &temp1, &xmin, &xmax, &temp2, &ymin,
                    &ymax);
                active->XRANGE = (int)(xmax - xmin) + 1;
                active->datahistory = malloc (sizeof (double) * active->XRANGE *
                    traces);
                active->data = malloc (sizeof (double) * traces);
                active->min = 10000;
                active->max = -10000;
                }
            else
                active = active->nextplot;
        else {                                      /* found a match */
            match = 1;
            active->count = active->count + 1;
            }
        } /* end while */


    /*  Copy the trace points into our buffer called data */

    for (i = 0; i < traces; i++)
        active->data[i] = yArray[i];

    /*  If the number of points plotted so far is greater than the
        XRANGE (width of x axis), then we must copy the new data points
        into the history buffer while filtering out the old data that
        is off the screen.  If we're not off the chart yet, just copy
        the data points into the history buffer.                       */

    if (active->count >= active->XRANGE) {
        for (i = 1; i < (active->XRANGE * traces) - traces + 1; i++)
            active->datahistory[i-1] = active->datahistory[i+traces-1];
        for (i = (active->XRANGE - 1) * traces; i < active->XRANGE * traces; i++)
            active->datahistory[i] = active->data[i-traces*(active->XRANGE-1)];
        }
    else
        for (i = (active->count * traces); i < (active->count + 1) * traces; i++)
            active->datahistory[i] = active->data[i-traces*active->count];

    /*  Check to see if the y axis needs to be rescaled                */

    for (i = 0; i < traces; i++)
        if (active->data[i] < active->min)
            active->min = active->data[i];
    for (i = 0; i < traces; i++)
        if (active->data[i] > active->max)
            active->max = active->data[i];
    GetAxisRange (panel, control, &temp1, &xmin, &xmax, &temp2, &ymin, &ymax);
    if (active->min < ymin || active->max > ymax) {

        /*  Check if min is equal to max.  This only occurs for the
            first plot (because there is only one point, but max and min
            is the same).                                              */

        if (active->min == active->max)
            SetAxisRange (panel, control, VAL_NO_CHANGE, 0.0, 1.0, VAL_MANUAL,
                active->min, active->max + 0.1 * active->max + 1);
        else
            SetAxisRange (panel, control, VAL_NO_CHANGE, 0.0, 1.0, VAL_MANUAL,
                active->min, active->max);
        if (active->count > 0)
            if (active->count > active->XRANGE) {
                SetAxisRange (panel, control, VAL_MANUAL, xmin, 1, VAL_NO_CHANGE,
                    active->min, active->max);
                PlotStripChart (panel, control, active->datahistory,
                    (active->XRANGE - 1) * traces, 0, 0, VAL_DOUBLE);
                }
            else
                PlotStripChart (panel, control, active->datahistory,
                    active->count * traces, 0, 0, VAL_DOUBLE);

        PlotStripChart (panel, control, active->data, traces, 0, 0, VAL_DOUBLE);
        }
    else 
        if (active->count > active->XRANGE &&
            (active->count % active->XRANGE == (int)(active->XRANGE * 0.3) ||
            active->count % active->XRANGE == (int)(active->XRANGE * 0.8))) {
            MaxMin1D (active->datahistory, active->XRANGE * traces, &active->max,
                &temp1, &active->min, &temp2);
            if (active->max < ymax * 0.99 || active->min > xmin * 1.01) {
                if (active->min == active->max)
                    SetAxisRange (panel, control, VAL_MANUAL, (int)xmin + 1, 1,
                        VAL_MANUAL, active->min, active->max + 0.1 * active->max + 1);
                else
                    SetAxisRange (panel, control, VAL_MANUAL, (int)xmin + 1, 1,
                        VAL_MANUAL, active->min, active->max);
                PlotStripChart (panel, control, active->datahistory,
                    (active->XRANGE - 1) * traces, 0, 0, VAL_DOUBLE);
                ymin = active->min;
                ymax = active->max;
                }
            else
                PlotStripChart (panel, control, active->data, traces, 0, 0,
                    VAL_DOUBLE);
            }
        else
            PlotStripChart (panel, control, active->data, traces, 0, 0, VAL_DOUBLE);

    match = 0;
    return 0;
}

static void FreeStripBuffer (void)
{
    PLOTLINK    PLtemp, PLtemp2;

    PLtemp = headoflist;
    PLtemp2 = headoflist;

    if (actualnumplots <= 0)
        return;

    while (PLtemp) {
        free (PLtemp->data);
        free (PLtemp->datahistory);
        ClearStripChart (PLtemp->panel, PLtemp->control);
        PLtemp2 = PLtemp->nextplot;
        free (PLtemp);
        PLtemp = PLtemp2;
        actualnumplots--;
        }
        
    firstcall = TRUE;
    return;
}

