/*********************************************************************/
/*                                                                   */
/*  This program uses the User Interface Library functions for       */
/*  creating UIR file objects programmatically.  For example, you    */
/*  don't have to use the User Interface Editor to build your GUIs,  */
/*  you can use the NewPanel and NewCtrl functions (as shown in this */
/*  program) to build your front panels.                             */
/*                                                                   */
/*  You also can use the NewCtrl function to add controls to an      */
/*  existing UIR file.  You can build your GUI in the User Interface */
/*  Editor, and then customize or add more controls to the GUI with  */
/*  function calls in your program.                                  */
/*                                                                   */
/*********************************************************************/

#include <cvirte.h>    /* Needed if linking in external compiler; harmless otherwise */
#include <userint.h>
#include <ansi_c.h>

void BuildGUI(void);
int CVICALLBACK GenerateData (int panelHandle,
          int controlID, int event, void *callbackData, int
          eventData1, int eventData2);
int CVICALLBACK Shutdown (int panelHandle,
          int controlID, int event, void *callbackData, int
          eventData1, int eventData2);


int handle, goButton, readout, quitButton;


int main (int argc, char *argv[])
{
    if (InitCVIRTE (0, argv, 0) == 0)    /* Needed if linking in external compiler; harmless otherwise */
        return -1;    /* out of memory */
    
    BuildGUI();
    DisplayPanel (handle);
    RunUserInterface();

    return 0;
}

/*********************************************************************/
/*                                                                   */
/*  The BuildGUI function creates a panel with two Command Buttons   */
/*  and a numeric readout.  Notice that both command buttons are     */
/*  assigned a callback function with the InstallCtrlCallback        */
/*  function.  The SetCtrlAttribute functions sets the color of the  */
/*  command buttons.                                                 */
/*                                                                   */
/*********************************************************************/

void BuildGUI()
{

    handle = NewPanel (0, "Sample LabWindows/CVI Panel", 40, 30, 250, 450);

    goButton = NewCtrl (handle, CTRL_SQUARE_COMMAND_BUTTON, "Generate Data", 50, 50);
    InstallCtrlCallback (handle, goButton, GenerateData, 0);
    SetCtrlAttribute (handle, goButton, ATTR_CMD_BUTTON_COLOR, VAL_GREEN);

    quitButton = NewCtrl (handle, CTRL_SQUARE_COMMAND_BUTTON, "Quit", 150, 80);
    InstallCtrlCallback (handle, quitButton, Shutdown, 0);
    SetCtrlAttribute (handle, quitButton, ATTR_CMD_BUTTON_COLOR, VAL_RED);

    /* The ATTR_CLOSE_CTRL attribute specifies a control which will receive */
    /* a commit event when the panel is closed with the system menu         */
    SetPanelAttribute (handle, ATTR_CLOSE_CTRL, quitButton);

    readout = NewCtrl (handle, CTRL_NUMERIC, "Random Number", 100, 300);
    SetCtrlAttribute (handle, readout, ATTR_CTRL_MODE, VAL_INDICATOR);
}

int CVICALLBACK GenerateData (int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    double random_val;

    if (event == EVENT_COMMIT) {
        random_val = rand()/32767.0 * 100.0;
        SetCtrlVal (panel, readout, random_val);
        }
    return(0);
}

int CVICALLBACK Shutdown (int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    if (event == EVENT_COMMIT)
        QuitUserInterface(0);
    return(0);
}
