/************************************************************************/
/*                                                                      */
/*  This program introduces the callback function style of programming  */
/*  in LabWindows/CVI.  The program simply generates 100 random numbers */
/*  and plots them on a graph control.  The plotting is triggered by an */
/*  Acquire button on the front panel, and program execution is halted  */
/*  by the Quit button.                                                 */
/*                                                                      */
/************************************************************************/

#include <cvirte.h>    /* Needed if linking in external compiler; harmless otherwise */
#include <userint.h>
#include <ansi_c.h>
#include "callback.h"

int handle;
double datapoints[100];

/************************************************************************/
/*                                                                      */
/*  The main function is very simple when you use callbacks.  Notice    */
/*  that you only have to load and display the panels. The two callback */
/*  functions, AcquireData (executed when the Acquire button is clicked)*/
/*  and Shutdown (executed when the Quit button is clicked) are         */
/*  installed when you load the panels.  Calling RunUserInterface simply*/
/*  tells LabWindows/CVI to begin passing all event information from the*/
/*  to the front panels to the appropriate callback functions.          */
/*                                                                      */
/*  Open the callback.uir file to see how the callback functions are    */
/*  assigned.  If you edit the Acquire button in the UIR editor, you    */
/*  will see the AcquireData function name listed in the Callback       */
/*  Function control in the dialog box.                                 */
/*                                                                      */
/************************************************************************/

int main (int argc, char *argv[])
{
    if (InitCVIRTE (0, argv, 0) == 0)    /* Needed if linking in external compiler; harmless otherwise */
        return -1;    /* out of memory */
    
    handle = LoadPanel (0, "callback.uir", PANEL);
    DisplayPanel (handle);
    RunUserInterface ();

    return 0;
}

/*************************************************************************/
/*                                                                       */
/*  The AcquireData function is assigned to the Acquire button in the    */
/*  User Interface Editor.  The function is automatically prototyped in  */
/*  the header file callback.h generated by LabWindows/CVI.  This function*/
/*  is called by LabWindows/CVI whenever you click on the Acquire button */
/*  during program execution.                                            */
/*                                                                       */
/*  Notice that you must decide which event to respond to in the func-   */
/*  tion.  In this case, the datapoints are generated only when a        */
/*  COMMIT event is sensed (up mouse click).  LabWindows/CVI can recog-  */
/*  and pass many different event types, such as LEFT_CLICK, RIGHT_CLICK,*/
/*  LEFT_DOUBLE_CLICK, and so on.  To find out more about these events,  */
/*  refer to the event.prj example.                                      */
/*                                                                       */
/*************************************************************************/

int CVICALLBACK AcquireData(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    int i=0;

    if (event == EVENT_COMMIT) {
        for (i=0;i<100;i++)
            datapoints[i] = rand()/32767.0 * 100.0;
        PlotY (handle, PANEL_GRAPH, datapoints, 100, VAL_DOUBLE,
               VAL_THIN_LINE, VAL_EMPTY_SQUARE, VAL_SOLID, 1, VAL_RED);
        }
    return(0);
}

int CVICALLBACK Shutdown(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    if (event == EVENT_COMMIT)
        QuitUserInterface(0);
    return(0);
}
