/* Simple Arbitrary Drawing on a Canvas Control */

#define HELPTEXT "\
This sample program illustrates arbitrary drawing on a Canvas\n\
control. Click on any Draw Object buttons to randomly draw an\n\
object, or use the left mouse button to draw lines on the Canvas\n\
control."

#include <cvirte.h>     /* needed if linking executable in external compiler; harmless otherwise */
#include <userint.h>
#include <ansi_c.h>
#include "canvas.h"

static int hPanel;
static int nWidth, nHeight, nLeftDown;
static int nYPos, nXPos, prevCursor;

int main (int argc, char *argv[])
{
    if (InitCVIRTE (0, argv, 0) == 0)    /* Needed if linking in external compiler; harmless otherwise */
        return -1;    /* out of memory */
    
    if ((hPanel = LoadPanel (0, "canvas.uir", PANEL)) < 0)
        return -1;
        
    /* Get default values from the UIR */
    GetCtrlAttribute (hPanel, PANEL_CANVAS, ATTR_WIDTH, &nWidth);
    GetCtrlAttribute (hPanel, PANEL_CANVAS, ATTR_HEIGHT, &nHeight);
    
    GetMouseCursor (&prevCursor);
    SetMouseCursor (VAL_POINTING_FINGER_CURSOR);

    SetCtrlAttribute (hPanel, PANEL_PICTURERING, ATTR_VISIBLE, 0);
    
    DisplayPanel (hPanel);
    RunUserInterface ();
    
    SetMouseCursor (prevCursor);

    return 0;
}

int CVICALLBACK DrawToCanvas (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_TIMER_TICK:
            GetRelativeMouseState (hPanel, PANEL_CANVAS, &nXPos, &nYPos,
                                   &nLeftDown, 0, 0);
            if (!nLeftDown)
                SetCtrlAttribute (hPanel, PANEL_TIMER, ATTR_ENABLED, 0);
            else
                CanvasDrawLineTo (hPanel, PANEL_CANVAS,
                        MakePoint(nXPos, nYPos));
            break;
    }
    return 0;
}

int CVICALLBACK ScratchPad (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_LEFT_CLICK:
            SetCtrlAttribute (hPanel, PANEL_TIMER, ATTR_ENABLED, 1);
            GetRelativeMouseState (hPanel, PANEL_CANVAS, &nXPos, &nYPos,
                                   0, 0, 0);
            CanvasSetPenPosition (hPanel, PANEL_CANVAS,
                            MakePoint(nXPos, nYPos));
            break;
    }
    return 0;
}

/* Function to draw object as requested on the UIR panel */
int CVICALLBACK ExecuteCommand (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    if (event==EVENT_COMMIT) {
    
        
        int randTop, randLeft, randHeight, randWidth;
        int bitmapID;
        int x, y, i, value, color;
        Rect rectVal;
        Point pointVal1, pointVal2;
        Point pointArray[6];
        
        randTop    = 275*rand()/RAND_MAX + 5;
        randHeight =  50*rand()/RAND_MAX  + 30;
        
        randLeft    = 375*rand()/RAND_MAX + 5;
        randWidth   =  50*rand()/RAND_MAX + 30;

        rectVal = MakeRect (randTop, randLeft, randHeight, randWidth);
        
        switch(control) {
            case PANEL_CLEAR:
                CanvasClear (panel, PANEL_CANVAS, VAL_ENTIRE_OBJECT);
                break;
            default:
                GetCtrlVal (panel, control, &value);
                color = MakeColor(255*rand()/RAND_MAX, 255*rand()/RAND_MAX, 255*rand()/RAND_MAX);
                SetCtrlAttribute (hPanel, PANEL_CANVAS, ATTR_PEN_FILL_COLOR, color);
                
                color = MakeColor(255*rand()/RAND_MAX, 255*rand()/RAND_MAX, 255*rand()/RAND_MAX);
                SetCtrlAttribute (hPanel, PANEL_CANVAS, ATTR_PEN_COLOR, color);
                
                SetCtrlAttribute (hPanel, PANEL_CANVAS, ATTR_PEN_WIDTH, 5*rand()/RAND_MAX+1);
                
                /* Which button was pushed */
                switch (control) {
                    case PANEL_ARC:
                        CanvasDrawArc (panel, PANEL_CANVAS, rectVal, 0, 2700, VAL_DRAW_FRAME_AND_INTERIOR);
                        break;
                    case PANEL_BITMAP:
                        GetCtrlBitmap (panel, PANEL_PICTURERING, 4*rand()/RAND_MAX, &bitmapID);
                        CanvasDrawBitmap (panel, PANEL_CANVAS, bitmapID, VAL_ENTIRE_OBJECT, rectVal);
                        break;
                    case PANEL_LINE:
                        /* Decide points to draw from and to */
                        if (randTop%2==0) {
                            pointVal1 = MakePoint (randLeft, randTop); 
                            pointVal2 = MakePoint (randWidth, randHeight);
                        }    
                        else {
                            pointVal1 = MakePoint (randLeft, randTop+randHeight); 
                            pointVal2 = MakePoint (randLeft+randWidth, randTop);
                        }    
                        CanvasDrawLine (panel, PANEL_CANVAS, pointVal1, pointVal2);
                    
                        break;
                    case PANEL_OVAL:
                        CanvasDrawOval (panel, PANEL_CANVAS, rectVal, VAL_DRAW_FRAME_AND_INTERIOR);
                        break;
                    case PANEL_POINT:
                        CanvasStartBatchDraw (panel, PANEL_CANVAS);
                        /* Fill rectangle with random points */                                             
                        for (i=0;i<100;i++) {
                            y = randTop  + (randHeight*rand()/RAND_MAX);
                            x = randLeft + (randWidth *rand()/RAND_MAX);
                            CanvasDrawPoint (panel, PANEL_CANVAS, MakePoint(x,y));
                        }    
                        
                        CanvasEndBatchDraw (panel, PANEL_CANVAS);
                        break;
                    case PANEL_POLY:
                        /* Randomly decide on points to use */
                        value = (3*rand()/RAND_MAX) + 3;
                        for (i=0;i<value;i++) {
                            y = randTop  + ((randHeight+20)*rand()/RAND_MAX);
                            x = randLeft + ((randWidth+20) *rand()/RAND_MAX);
                            pointArray[i] = MakePoint(x,y);
                        }
                        CanvasDrawPoly (panel, PANEL_CANVAS, value, pointArray, 1,
                                        VAL_DRAW_FRAME_AND_INTERIOR);
                        break;
                    case PANEL_RECT:
                        CanvasDrawRect (panel, PANEL_CANVAS, rectVal, VAL_DRAW_FRAME_AND_INTERIOR);
                        break;
                    case PANEL_TEXT:
                        CanvasDrawText (panel, PANEL_CANVAS, "sample text", VAL_APP_META_FONT,
                                        rectVal, VAL_CENTER);
                        break;
                } /* switch (control) */
                
            break;
        } /* switch(control) */
        
    } /* if (event==EVENT_COMMIT) */
    return 0;
}

int CVICALLBACK Quit (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            QuitUserInterface (0);
            break;
    }
    return 0;
}

int CVICALLBACK Help (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            MessagePopup ("Simple Canvas Control", HELPTEXT);
            break;
    }
    return 0;
}
