#include "chart.h"

/************************************************************************/
/*                                                                      */
/*  This program shows you how to perform a task continuously while     */
/*  waiting for user interface events to occur on your front panels.    */
/*  You can use a timer control to cause a function to be called        */
/*  continuously while CVI waits to process any events from controls    */
/*  through other callback functions.  In this example, ProcessLoop     */
/*  will be called continuously.  If the Plot flag is set, ProcessLoop  */
/*  will generate fake data and plot it on a strip chart.  The timer    */
/*  control is on the panel, but is only visible in the UIR editor.     */
/*  The callback function for the timer control is ProcessLoop.         */
/*                                                                      */
/************************************************************************/

#include <cvirte.h>    /* Needed if linking in external compiler; harmless otherwise */
#include <userint.h>
#include <ansi_c.h>
#include "chart.h"

#define TRUE  1
#define FALSE 0


int handle, plotdata = FALSE;
double datapoints[3];

int main (int argc, char *argv[])
{
    if (InitCVIRTE (0, argv, 0) == 0)    /* Needed if linking in external compiler; harmless otherwise */
        return -1;    /* out of memory */
    
    handle = LoadPanel (0, "chart.uir", PANEL);
    DisplayPanel (handle);
    RunUserInterface ();

    return 0;
}

/****************************************************************************/
/*                                                                          */
/*  ProcessLoop checks the plotdata flag.  The plotdata flag is set         */
/*  when the user clicks on the Plot button.  If the flag is set,           */
/*  ProcessLoop will generate and plot data on the strip chart.  The        */
/*  rate at which ProcessLoop is called is set by the interval of the       */
/*  timer control. The interval can be set in the timer control editing     */
/*  dialog in the User Interface Editor, or by changed the ATTR_INTERVAL    */
/*  attribute of the timer control in a program using SetCtrlAttribute().   */
/*  If the interval is set to zero, the timer control will call             */
/*  ProcessLoop as fast as possible.                                        */
/*                                                                          */
/****************************************************************************/


    /* callback function for the timer control */
int CVICALLBACK ProcessLoop (int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_TIMER_TICK:
           if (plotdata == TRUE) {
                datapoints[0] = 100.0 * rand() / RAND_MAX;
                datapoints[1] = 100.0 * rand() / RAND_MAX;
                datapoints[2] = 100.0 * rand() / RAND_MAX;

                PlotStripChart (handle, PANEL_CHART, datapoints, 3, 0, 0, VAL_DOUBLE);
                }
            break;
    }
    
    return 0;
}

int CVICALLBACK PlotData(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    if (event == EVENT_COMMIT)
        plotdata = TRUE;
    return(0);
}

int CVICALLBACK Shutdown(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    if (event == EVENT_COMMIT)
        QuitUserInterface(0);
    return(0);
}
