/*********************************************************************/
/*                                                                   */
/*  This program demonstrates the use of free form and snap to       */
/*  point graph cursors.                                             */
/*                                                                   */
/*********************************************************************/

#include <cvirte.h>    /* Needed if linking in external compiler; harmless otherwise */
#include <userint.h>
#include <ansi_c.h>
#include "cursors.h"

int handle;
double datapoints[100];

int main (int argc, char *argv[])
{
    if (InitCVIRTE (0, argv, 0) == 0)    /* Needed if linking in external compiler; harmless otherwise */
        return -1;    /* out of memory */
    
    handle = LoadPanel (0, "cursors.uir", PANEL);
    DisplayPanel (handle);
    RunUserInterface();

    return 0;
}

/*********************************************************************/
/*                                                                   */
/*  The PlotData function generates random data and plots it on the  */
/*  graph control.                                                   */
/*                                                                   */
/*********************************************************************/

int CVICALLBACK PlotData(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    int i;

    if (event == EVENT_COMMIT) {
        DeleteGraphPlot (handle, PANEL_GRAPH, -1, 1);
        for (i=0;i<100;i++)
            datapoints[i] = rand()/32767.0 * 100.0;
        PlotY (handle, PANEL_GRAPH, datapoints, 100, VAL_DOUBLE,
            VAL_THIN_LINE, VAL_EMPTY_SQUARE, VAL_SOLID, 1, VAL_RED);
    }
    return(0);
}

/********************************************************************/
/*                                                                  */
/*  The RestoreGraph function restores the graph control to its     */
/*  original state, before any zooming was done.  This is done      */
/*  simply by resetting the X and Y-axis ranges to their original   */
/*  values to rescale the graph.                                    */
/*                                                                  */
/********************************************************************/

int CVICALLBACK RestoreGraph(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{

    if (event == EVENT_COMMIT)
        SetAxisRange (handle, PANEL_GRAPH, VAL_MANUAL, 0, 100, VAL_MANUAL, 0, 100);
    return(0);
}

/*********************************************************************/
/*                                                                   */
/*  The UpdateMarker function updates the X and Y-coordinate of the  */
/*  marker when it is moved (the marker is the cursor on the graph   */
/*  with a short cross.  This function is called whenever the graph  */
/*  cursors are moved.  When a graph control has cursors, it can be  */
/*  used as a hot input control.                                     */
/*                                                                   */
/*********************************************************************/

int CVICALLBACK UpdateMarker(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    double x,y;
    int plothandle, index;

    if (event == EVENT_COMMIT) {
        GetGraphCursor (handle, PANEL_GRAPH, 2, &x, &y);
        GetGraphCursorIndex (handle, PANEL_GRAPH, 2, &plothandle, &index);
        SetCtrlVal (handle, PANEL_XREADOUT, x);
        SetCtrlVal (handle, PANEL_YREADOUT, y);
        SetCtrlVal (handle, PANEL_INDEX, index);
        }
    return(0);
}

/***********************************************************************/
/*                                                                     */
/*  The ZoomIn function simulates a zoom operation on the graph by     */
/*  rescaling it.  When the Zoom button is pushed, this function gets  */
/*  the current position of the two cursors and rescales the graph     */
/*  based on their position.  The graph cursors define the zoom region.*/
/*  In this example, zooming is done only in the X direction.          */
/*                                                                     */
/***********************************************************************/

int CVICALLBACK ZoomIn(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    double x1, x2, y1, y2;
    double temp;
    if (event == EVENT_COMMIT) {
        GetGraphCursor (handle, PANEL_GRAPH, 1, &x1, &y1);
        GetGraphCursor (handle, PANEL_GRAPH, 3, &x2, &y2);
        if (x1 > x2)
        {
            temp = x1;
            x1 = x2;
            x2 = temp;
        }
        if (y1 > y2)
        {
            temp = y1;
            y1 = y2;
            y2 = temp;
        }
        SetAxisRange (handle, PANEL_GRAPH, VAL_MANUAL, x1, x2, VAL_MANUAL, y1,
                      y2);
    }
    return(0);
}

int CVICALLBACK Shutdown(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    if (event == EVENT_COMMIT)
        QuitUserInterface(0);
    return(0);
}

