/* Simple Arbitrary Drawing on a Canvas Control */

#define HELPTEXT "\
This sample program illustrates arbitrary drawing on a Canvas control. Use\n\
the left mouse button to draw lines on the Canvas control.  You can also\n\
change the pen color and width."

#include <cvirte.h>     /* needed if linking executable in external compiler; harmless otherwise */
#include <userint.h>
#include "drawpad.h"

static int hPanel;
static int nWidth, nHeight, nLeftDown;
static int nYPos, nXPos, prevCursor;

int main (int argc, char *argv[])
{
    if (InitCVIRTE (0, argv, 0) == 0)    /* Needed if linking in external compiler; harmless otherwise */
        return -1;    /* out of memory */
    
    if ((hPanel = LoadPanel (0, "drawpad.uir", PANEL)) < 0)
        return -1;
        
    /* Get default values from the UIR */
    GetCtrlAttribute (hPanel, PANEL_CANVAS, ATTR_WIDTH, &nWidth);
    GetCtrlAttribute (hPanel, PANEL_CANVAS, ATTR_HEIGHT, &nHeight);
    
    GetMouseCursor (&prevCursor);
    SetMouseCursor (VAL_POINTING_FINGER_CURSOR);
    
    DisplayPanel (hPanel);
    RunUserInterface ();
    
    SetMouseCursor (prevCursor);

    return 0;
}

int CVICALLBACK DrawToCanvas (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_TIMER_TICK:
            GetRelativeMouseState (hPanel, PANEL_CANVAS, &nXPos, &nYPos,
                                   &nLeftDown, 0, 0);
            if (!nLeftDown)
                SetCtrlAttribute (hPanel, PANEL_TIMER, ATTR_ENABLED, 0);
            else
                CanvasDrawLineTo (hPanel, PANEL_CANVAS,
                        MakePoint(nXPos, nYPos));
            break;
    }
    return 0;
}

int CVICALLBACK ScratchPad (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_LEFT_CLICK:
            SetCtrlAttribute (hPanel, PANEL_TIMER, ATTR_ENABLED, 1);
            GetRelativeMouseState (hPanel, PANEL_CANVAS, &nXPos, &nYPos,
                                   0, 0, 0);
            CanvasSetPenPosition (hPanel, PANEL_CANVAS,
                            MakePoint(nXPos, nYPos));
            break;
    }
    return 0;
}

int CVICALLBACK SetColor (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    int nPenColor;
    
    switch (event) {
        case EVENT_COMMIT:
            GetCtrlVal (hPanel, PANEL_COLORNUM, &nPenColor);
            SetCtrlAttribute (hPanel, PANEL_CANVAS, ATTR_PEN_COLOR, nPenColor);
            break;
    }
    return 0;
}

int CVICALLBACK PixelWidth (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    unsigned short nPixelWidth;
    
    switch (event) {
        case EVENT_COMMIT:
            GetCtrlVal (hPanel, PANEL_NUMERIC, &nPixelWidth);
            SetCtrlAttribute (hPanel, PANEL_CANVAS, ATTR_PEN_WIDTH, nPixelWidth);
            break;
    }
    return 0;
}

int CVICALLBACK Quit (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            QuitUserInterface (0);
            break;
    }
    return 0;
}

int CVICALLBACK Help (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            MessagePopup ("Simple Drawing Pad", HELPTEXT);
            break;
    }
    return 0;
}

