
#include <cvirte.h>    /* Needed if linking in external compiler; harmless otherwise */
#include <userint.h>
#include <stdio.h>
#include <formatio.h>
#include "events.h"

/*************************************************************************/
/*                                                                       */
/*  This example illustrates how to recognize many of the low-level      */
/*  events and event-related information that LabWindows/CVI passes from */
/*  the user interface to your program.  The PanelCallback function will */
/*  receive the events generated during runtime, and display the event   */
/*  information on the UIR.  Notice that for all mouse click events, the */
/*  X and Y coordinate position of the mouse are also updated on the     */
/*  screen.  For KEYPRESS events, the ASCII key code of the key typed is */
/*  displayed, instead of the mouse X and Y coordinates.                 */
/*************************************************************************/


void UpdateTextBoxClick (char* event_gen, int eventDat1, int eventDat2);
void UpdateTextBox (char* event_gen);

int handle; 


int main (int argc, char *argv[])
{
    if (InitCVIRTE (0, argv, 0) == 0)    /* Needed if linking in external compiler; harmless otherwise */
        return -1;    /* out of memory */
    
    handle = LoadPanel (0, "events.uir", PANEL);
    DisplayPanel (handle);
    RunUserInterface();

    return 0;
}


/*  This is the Panel Callback.  Any events that occur on the panel will call this
    function. The switch statement checks for the specific event that occurred.    
*/
int CVICALLBACK PanelCallback(int panel, int event, void *callbackData, int eventData1, int eventData2)
{
    char keypushed[100];

    switch (event) {
        case EVENT_LEFT_CLICK :
            UpdateTextBoxClick("Left Click                 Panel            ", eventData1, eventData2);
            break;
        case EVENT_RIGHT_CLICK :
            UpdateTextBoxClick("Right Click               Panel            ", eventData1, eventData2);
            break;
        case EVENT_LEFT_DOUBLE_CLICK :
            UpdateTextBoxClick("Left Double Click      Panel            ", eventData1, eventData2);
            break;
        case EVENT_RIGHT_DOUBLE_CLICK :
            UpdateTextBoxClick("Right Double Click    Panel            ", eventData1, eventData2);
            break;
        case EVENT_KEYPRESS :
            if ((eventData1 & VAL_ASCII_KEY_MASK) != 0){
                sprintf (keypushed, "Keypress detected     %c was pressed \n", eventData1);
                SetCtrlVal (panel, PANEL_TEXTBOX, keypushed);
            }
            break;
        case EVENT_PANEL_SIZE :
            UpdateTextBox("Panel Resize            Panel");
            break;
        case EVENT_PANEL_MOVE :
            UpdateTextBox("Panel Moved            Panel");
            break;
        case EVENT_CLOSE :
            UpdateTextBox("Panel Close Event     Panel");
            break;
        }

    return 0;
}

/*  Control CallBack for the Slide.    */
int CVICALLBACK SlideCallback(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
double  interval;

    switch (event) {
        case EVENT_GOT_FOCUS :
            UpdateTextBox("Got Focus                Slide Control");
            break;
        case EVENT_LOST_FOCUS :
            UpdateTextBox("Lost Focus               Slide Control");
            break;
        case EVENT_VAL_CHANGED :
            UpdateTextBox("Value Changed               Slide Control");
            break;
        case EVENT_COMMIT :
            UpdateTextBox("Control Committed     Slide Control");
            GetCtrlVal (panel, PANEL_SLIDE, &interval);
            SetCtrlAttribute (panel, PANEL_TIMER, ATTR_INTERVAL, interval);
            break;
        }
        
    return 0;
}

/*  Control CallBack for the Toggle Switch.    */
int CVICALLBACK ToggleCallback(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    int timerEventsOn;
    
    switch (event) {
        case EVENT_LEFT_CLICK :
            UpdateTextBoxClick("Left Click                 Toggle Switch", eventData1, eventData2);
            break;
        case EVENT_RIGHT_CLICK :
            UpdateTextBoxClick("Right Click               Toggle Switch", eventData1, eventData2);
            break;
        case EVENT_LEFT_DOUBLE_CLICK :
            UpdateTextBoxClick("Left Double Click      Toggle Switch ", eventData1, eventData2);
            break;
        case EVENT_RIGHT_DOUBLE_CLICK :
            UpdateTextBoxClick("Right Double Click    Toggle Switch ", eventData1, eventData2);
            break;
        case EVENT_GOT_FOCUS :
            UpdateTextBox("Got Focus                Toggle Switch");
            break;
        case EVENT_LOST_FOCUS :
            UpdateTextBox("Lost Focus               Toggle Switch");
            break;
        case EVENT_VAL_CHANGED :
            UpdateTextBox("Value Changed               Toggle Switch");
            break;
        case EVENT_COMMIT :
            UpdateTextBox("Control Committed     Toggle Switch");
            
            GetCtrlVal(panel, control, &timerEventsOn);
            SetCtrlAttribute(panel, PANEL_TIMER, ATTR_ENABLED, timerEventsOn);
            break;
        }
        
    return 0;
}

/*  This is the callback for the timer control.  It is called at
    which is called at a rate determined by the interval attribute
    of the timer control. In this program, the slide determines 
    this rate.                                                              
*/

int CVICALLBACK TimerCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_TIMER_TICK:
             UpdateTextBox("Timer Tick Event      Timer Control Callback");
            break;                        
    }
    
    return 0;
}

/*  Control CallBack for the Quit Program button on Event Generation Panel  */
int CVICALLBACK Shutdown1(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT   :
             QuitUserInterface (0);
            break;
        case EVENT_GOT_FOCUS :
            UpdateTextBox("Got Focus                Quit Button");
            break;
        case EVENT_LOST_FOCUS :
            UpdateTextBox("Lost Focus               Quit Button");
            break;
        }
        
    return 0;
}

/*  Control CallBack for the Quit Program button on Status Panel */
int CVICALLBACK Shutdown2(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    if (event == EVENT_COMMIT)
        QuitUserInterface (0);
        
    return 0;
}

/*  These functions update the status Text Box with information about the event.  */
void UpdateTextBoxClick (char* event_gen, int eventDat1, int eventDat2)
{
    char display_str[100];
    
    Fmt(display_str, "%s<%s          Y coordinate= %i,  X coordinate= %i\n",event_gen, eventDat1, eventDat2);
    SetCtrlVal (handle, PANEL_TEXTBOX, display_str);
}

void UpdateTextBox (char* event_gen)
{
    char display_str[100];

    Fmt(display_str, "%s<%s\n", event_gen);
    SetCtrlVal (handle, PANEL_TEXTBOX, display_str);
}
