
#include <cvirte.h>    /* Needed if linking in external compiler; harmless otherwise */
#include <userint.h>
#include <ansi_c.h>
#include <analysis.h>
#include "getusrev.h"

/*  This example shows the event loop programming model.
    Although this model is not the preferred programming model
    in CVI (the callback model is much more flexible and easy to
    implement), it can still be used. LabWindows DOS programs 
    used this model exclusively therefore, any program that is 
    translated from LW/DOS will use the event loop model.  It
    is up to the user to reformat the program into the callback
    model if so desired.  However, callbacks and GetUserEvent
    loops may be freely mixed.
*/

#define TRUE 1
#define FALSE 0
#define NOWAIT 0  /* get user event without waiting */

#define PI   3.1415926535897932384626433832795028841971


int pnl_handle;
static double waveform[100];


void GenerateSineWave (int numPoints, double amplitude,
                       double cycles, double *waveform);


int main (int argc, char *argv[])
{
    int handle_returned, controlID_returned;
    int done = FALSE;
    double cycles;

    if (InitCVIRTE (0, argv, 0) == 0)    /* Needed if linking in external compiler; harmless otherwise */
        return -1;    /* out of memory */
    
    pnl_handle = LoadPanel (0, "getusrev.uir", PNL);
    DisplayPanel (pnl_handle);

    while (done == FALSE)
    {
        GetUserEvent (NOWAIT, &handle_returned, &controlID_returned);

        switch (controlID_returned) {

            case PNL_GENERATE:
                GetCtrlVal (pnl_handle, PNL_CYCLES, &cycles);
                GenerateSineWave (100, 1.0, cycles, waveform);
                PlotY (pnl_handle, PNL_GRAPH, waveform, 100, VAL_DOUBLE,
                       VAL_THIN_LINE, VAL_EMPTY_SQUARE, VAL_SOLID, 1, VAL_RED);
                break;

            case PNL_DELPLOTS:
                DeleteGraphPlot (pnl_handle, PNL_GRAPH, -1, 1);
                break;

            case PNL_QUIT:
                done = TRUE;
            break;
            }
        }

    DiscardPanel (pnl_handle);

    return 0;
}


void GenerateSineWave (int numPoints, double amplitude,
                       double cycles, double *waveform)
{
    double x, dx;
    int i;
    
    x = 0.0;
    dx = (2.0 * PI) / (double)(numPoints - 1);
    for (i = 0 ; i < numPoints ; i++)
    {
        waveform[i] = sin (cycles * x);
        x += dx;
    }
}
