
#include <cvirte.h>    /* Needed if linking in external compiler; harmless otherwise */
#include <userint.h>
#include <ansi_c.h>
#include "graphs.h"

int handle;
double datapoints[100];

int main (int argc, char *argv[])
{
    int i;

    if (InitCVIRTE (0, argv, 0) == 0)    /* Needed if linking in external compiler; harmless otherwise */
        return -1;    /* out of memory */
    
    /**  Generate some random data first **/
    for (i=0; i<100; i++)
        datapoints[i] = 100.0 * rand() / RAND_MAX;

    handle = LoadPanel (0, "graphs.uir", PANEL);
    DisplayPanel (handle);
    RunUserInterface ();

    return 0;
}

/**********************************************************************/
/*                                                                    */
/*  The PlotData function gets the values of the color and plotstyle  */
/*  controls, and displays the random datapoints array on the graph   */
/*  control based on the user inputs. This function is called by the  */
/*  Plot Data button, the Line Style Ring Control, and the Trace Color*/
/*  color ring control.                                               */
/**********************************************************************/

int CVICALLBACK PlotData(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    int color, style, i;

    if (event == EVENT_COMMIT) {
        GetCtrlVal (handle, PANEL_TRACECOLOR, &color);
        GetCtrlVal (handle, PANEL_STYLE, &style);
        DeleteGraphPlot (handle, PANEL_GRAPH, -1, 1);
        PlotY (handle, PANEL_GRAPH, datapoints, 100, VAL_DOUBLE, style,
               VAL_EMPTY_SQUARE, VAL_SOLID, 1, color);
        }
        
    return 0;
}

/**********************************************************************/
/*                                                                    */
/*  The PlotShape function gets the values of the color, shape, and   */
/*  fill controls, and plots shapes on the graph control based on     */
/*  the user inputs.  This function uses the LabWindows/CVI plotting  */
/*  functions for displaying shapes, points, and text on a graph      */
/*  control.                                                          */
/*                                                                    */
/**********************************************************************/

int CVICALLBACK PlotShape(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    int color, color2, shape, fill;

    if (event == EVENT_COMMIT) {
        GetCtrlVal (handle, PANEL_FILLCOLOR, &color);
        GetCtrlVal (handle, PANEL_SHAPE, &shape);
        GetCtrlVal (handle, PANEL_FILLMODE, &fill);
        if (fill)
            color2 = VAL_TRANSPARENT;
        else
            color2 = color;
        DeleteGraphPlot (handle, PANEL_GRAPH, -1, 1);
        switch (shape) {
            case 0 :
                PlotRectangle (handle, PANEL_GRAPH, 20, 20, 80, 80, color, color2);
                break;
            case 1 :
                PlotOval (handle, PANEL_GRAPH, 20, 40, 80, 80, color, color2);
                break;
            case 2 :
                PlotArc (handle, PANEL_GRAPH, 20, 20, 80, 80, 1500, 900, color, color2);
                break;
            case 3 :
                PlotPoint (handle, PANEL_GRAPH, 44, 66, VAL_EMPTY_SQUARE, color);
                break;
            case 4 :
                PlotText (handle, PANEL_GRAPH, 30, 40, "TEXT", VAL_MENU_META_FONT,
                      color, VAL_TRANSPARENT);
                break;
            }

        }
        
    return 0;
}


int CVICALLBACK Shutdown(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    if (event == EVENT_COMMIT)
        QuitUserInterface(0);
        
    return 0;
}
