/*  This program displays a string control and lets the user change its behaviour
    by selecting check box options.  The strings control's behaviour is changed
    by responding to key events in the string controls callback function.
*/

#include <cvirte.h>    /* Needed if linking in external compiler; harmless otherwise */
#include <ansi_c.h>
#include <userint.h>
#include "keyfiltr.h"

static int panelHandle;

int main (int argc, char *argv[])
{
    if (InitCVIRTE (0, argv, 0) == 0)    /* Needed if linking in external compiler; harmless otherwise */
        return -1;    /* out of memory */
    
    panelHandle = LoadPanel (0, "keyfiltr.uir", PANEL);
    DisplayPanel (panelHandle);
    RunUserInterface ();

    return 0;
}

int CVICALLBACK StringCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    int forceToUpperCase;
    int changePunctuationToSpaces;
    int ignoreNumbers;
    int disableCursorKeys;
    int asciiCode;
    int virtualKey;         /* cursor keys, function keys, esc, enter, etc.. are virtual keys */
    int swallowEvent = 0;   /* 0 means DO NOT swallow (ignore) the event, only mouse and key event can be swallowed */
    
    switch (event)
        {
        case EVENT_KEYPRESS:
                /*  get the state of the options from the check boxes on the panel */
            GetCtrlVal (panelHandle, PANEL_FORCE_TO_UPPER,      &forceToUpperCase);
            GetCtrlVal (panelHandle, PANEL_PUNCT_IS_SPACES,     &changePunctuationToSpaces);
            GetCtrlVal (panelHandle, PANEL_IGNORE_NUMBERS,      &ignoreNumbers);
            GetCtrlVal (panelHandle, PANEL_DISABLE_CURSOR_KEYS, &disableCursorKeys);

            /*  eventData1 has the keypress value represented by a 4-byte integer consisting 
                of 3 fields, 0x00MMVVAA: MM = the modifier key, VV = the virtual key, -+*
                AA = the ASCII key.  eventData2 is a pointer to an integer which hold
                the actual key value.  This pointer can be used to change the value of
                the key before the control processes it. */
        
            asciiCode = eventData1 & VAL_ASCII_KEY_MASK;    /* key masks are defined in userint.h */
            virtualKey = eventData1 & VAL_VKEY_MASK;
            
                /*  if the key was an ascii key and the menukey (ctrl on the PC) was not pressed */
            if (asciiCode != 0 && (eventData1 & VAL_MENUKEY_MODIFIER) == 0)
                {
                if (forceToUpperCase && islower(asciiCode))
                    *(int *)eventData2 = toupper(asciiCode);
                    
                if (changePunctuationToSpaces && ispunct(asciiCode))
                    *(int *)eventData2 = ' ';
                    
                if (ignoreNumbers && isdigit(asciiCode))
                    swallowEvent = 1;
                }
            else
            if (disableCursorKeys && virtualKey > 0)  /* check for and swallow cursor key events */
                {
                switch (virtualKey)
                    {
                    case VAL_UP_ARROW_VKEY:
                    case VAL_DOWN_ARROW_VKEY:
                    case VAL_LEFT_ARROW_VKEY:
                    case VAL_RIGHT_ARROW_VKEY:
                    case VAL_HOME_VKEY:
                    case VAL_END_VKEY:
                        swallowEvent = 1;
                    }
                }
            break;
        }
        
    return swallowEvent;
}

int CVICALLBACK QuitCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event)
        {
        case EVENT_COMMIT:
            QuitUserInterface (0);
            break;
        }
    return 0;
}

int CVICALLBACK HelpCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event)
        {
        case EVENT_COMMIT:
            MessagePopup("Key Event Filtering Example - Help",
                "This sample program demonstrates how to intercept and respond to\n"
                "events to modify a control's behaviour.  Click on the check boxes\n"
                "to change how the string control responds to key events.\n");
            break;
        }
    return 0;
}
