/**********************************************************************/
/*                                                                    */
/*  This program demonstrates list box item selection and the use of  */
/*  character escape sequences to control item label color and        */
/*  positioning.                                                      */
/*                                                                    */
/**********************************************************************/

#include <cvirte.h>    /* Needed if linking in external compiler; harmless otherwise */
#include <userint.h>
#include <ansi_c.h>
#include "listdelx.h"

int handle;
int bgcolor, fgcolor;
int numVisibleItems;
int listboxWidth, firstListboxItemTop;
int leftListboxEdge, rightListboxEdge;
int vertDivision1, vertDivision2;
int textHeight, textWidth;
int numItems = 0;
int selectionIndex = 0;
char blankItem[40];
char text[60], buf[256];

typedef enum { LeftJustify, RightJustify, CenterJustify } justType;

char *AppendJustification(char *buf, int pix, justType just)
{
    char escSeq[20];

    switch (just) {
        case LeftJustify:   sprintf (escSeq, "\033p%dl", pix);  break;
        case RightJustify:  sprintf (escSeq, "\033p%dr", pix);  break;
        case CenterJustify: sprintf (escSeq, "\033p%dc", pix);  break;
    }
    return strcat (buf, escSeq);
}

char *AppendVLine(char *buf)
{
    return strcat (buf, "\033vline");
}

char *AppendColorSpec(char *buf, int bg, int fg)
{
    char colorSpec[24];

    sprintf(colorSpec, "\033bg%06X\033fg%06X", bg, fg);
    return strcat (buf, colorSpec);
}

int CVICALLBACK ListBoxSelect(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    if ((event == EVENT_LEFT_CLICK) || (event == EVENT_LEFT_DOUBLE_CLICK)) {
        /* eventData2 is x pixel value of click */
        if (eventData2 < leftListboxEdge || rightListboxEdge < eventData2)
            return 0;
        /* eventData1 is y pixel value of click */
        if (eventData1 < firstListboxItemTop)
            return 0;
        GetCtrlAttribute (panel, PANEL_LIST, ATTR_FIRST_VISIBLE_LINE, &selectionIndex);
        selectionIndex += (eventData1 - firstListboxItemTop) / textHeight;
        if (selectionIndex >= numItems)
            selectionIndex = numItems ? numItems - 1 : 0;
        /* explicitly set active list box item highlight and swallow event */
        SetCtrlIndex (panel, PANEL_LIST, selectionIndex);
        SetActiveCtrl (panel, PANEL_LIST);
        return 1;
    }
    return 0;
}

int CVICALLBACK AddCombo(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    if (event == EVENT_COMMIT) {
        GetCtrlVal (handle, PANEL_TEXT, text);
        GetCtrlVal (handle, PANEL_BACKGROUND, &bgcolor);
        GetCtrlVal (handle, PANEL_FOREGROUND, &fgcolor);

        sprintf (buf, "  %#08X  ", bgcolor);
        AppendJustification (buf, vertDivision1, LeftJustify);
        AppendVLine (buf);
        sprintf (strrchr (buf, 0), "  %#08X  ", fgcolor);
        AppendJustification (buf, vertDivision2, LeftJustify);
        AppendVLine (buf);
        AppendVLine (buf);
        AppendColorSpec (buf, bgcolor, fgcolor);
        strcat (buf, text);

        InsertListItem (handle, PANEL_LIST, selectionIndex, buf, 0);
        if (++numItems <= numVisibleItems)
            DeleteListItem (handle, PANEL_LIST, numVisibleItems, 1);
    }

    return(0);
}

int CVICALLBACK DeleteCombo(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    if (event == EVENT_COMMIT && numItems) {
        if (selectionIndex >= numItems)
            selectionIndex = numItems ? numItems - 1 : 0;
        SetCtrlIndex (panel, PANEL_LIST, selectionIndex);
        DeleteListItem (handle, PANEL_LIST, selectionIndex, 1);
        if (--numItems < numVisibleItems)
            InsertListItem (handle, PANEL_LIST, numItems, blankItem, 0);
    }
    return(0);
}


int CVICALLBACK Shutdown(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    if (event == EVENT_COMMIT)
        QuitUserInterface(0);
    return(0);
}

int main (int argc, char *argv[])
{
    int n;

    if (InitCVIRTE (0, argv, 0) == 0)    /* Needed if linking in external compiler; harmless otherwise */
        return -1;    /* out of memory */
    
    handle = LoadPanel (0, "listdelx.uir", PANEL);
    SetPanelAttribute (handle, ATTR_CLOSE_CTRL, PANEL_QUIT);

    /* get list box attributes */
    GetCtrlAttribute(handle, PANEL_LIST, ATTR_TOP, &firstListboxItemTop);
    firstListboxItemTop += 4; /* skip list box frame */
    GetCtrlAttribute (handle, PANEL_LIST, ATTR_WIDTH, &listboxWidth);
    listboxWidth -= (8 + VAL_LARGE_SCROLL_BARS);    /* subtract left and right list box frame and scroll bar */
    GetCtrlAttribute(handle, PANEL_LIST, ATTR_LEFT, &leftListboxEdge);
    leftListboxEdge += 4;   /* skip list box frame */
    rightListboxEdge = leftListboxEdge + listboxWidth;
    GetCtrlAttribute (handle, PANEL_LIST, ATTR_VISIBLE_LINES, &numVisibleItems);
    CreateMetaFont ("Editor14", VAL_EDITOR_FONT, 14, 0, 0, 0, 0);
    GetTextDisplaySize ("  0xFFFFFF  ", "Editor14", &textHeight, &textWidth);

    /* compute pixel positions for vertical line divisions */
    vertDivision1 = textWidth;
    vertDivision2 = textWidth * 2 + 6;

    /* initialize list box with vertical line divisions */
    sprintf (blankItem, "\033p%dl\033vline\033p%dl\033vline\033vline", vertDivision1, vertDivision2);
    for (n = 0; n < numVisibleItems; n++) {
        InsertListItem (handle, PANEL_LIST, n, blankItem, 0);
    }

    DisplayPanel (handle);
    RunUserInterface ();

    return 0;
}

