/************************************************************************/
/*                                                                      */
/*  This example illustrates how to respond to menubars with callback   */
/*  functions in LabWindows/CVI.  The MenuCallback function is called   */
/*  by the items in the Configuration and Tests menus.                  */
/*  The InstrConfig function is called by the items in the Instruments  */
/*  menu.  The callback functions simply update the string displays on  */
/*  the panel based on the menu selections.                             */
/*                                                                      */
/************************************************************************/

#include <cvirte.h>    /* Needed if linking in external compiler; harmless otherwise */
#include <userint.h>
#include "menus.h"

int main (int argc, char *argv[])
{
    int handle;

    if (InitCVIRTE (0, argv, 0) == 0)    /* Needed if linking in external compiler; harmless otherwise */
        return -1;    /* out of memory */
    
    handle = LoadPanel (0, "menus.uir", PANEL);
    DisplayPanel (handle);
    RunUserInterface ();

    return 0;
}

void CVICALLBACK MenuCallback(int menubar, int menuItem, void *callbackData, int panel)
{
    switch (menuItem) {
        case MENU_CONFIG_SET1 :
            SetCtrlVal (panel, PANEL_CONFIG, "Setting 1");
            break;
        case MENU_CONFIG_SET2 :
            SetCtrlVal (panel, PANEL_CONFIG, "Setting 2");
            break;
        case MENU_TESTS_TEST1 :
            SetCtrlVal (panel, PANEL_TEST, "Test 1");
            break;
        case MENU_TESTS_TEST2 :
            SetCtrlVal (panel, PANEL_TEST, "Test 2");
            break;
        }
}

void CVICALLBACK InstrConfig(int menubar, int menuItem, void *callbackData, int panel)
{
    switch (menuItem) {
        case MENU_INSTR_SCOPES_TEK640 :
            SetCtrlVal (panel, PANEL_INSTR, "Tek TDS 640");
            break;
        case MENU_INSTR_SCOPES_LEC9400A :
            SetCtrlVal (panel, PANEL_INSTR, "LeCroy 9400A");
            break;
        case MENU_INSTR_SCOPES_HP54501 :
            SetCtrlVal (panel, PANEL_INSTR, "HP 54501");
            break;
        case MENU_INSTR_METERS_FL45 :
            SetCtrlVal (panel, PANEL_INSTR, "Fluke 45");
            break;
        case MENU_INSTR_METERS_KEI199 :
            SetCtrlVal (panel, PANEL_INSTR, "Keithley 199");
            break;
        case MENU_INSTR_METERS_HP3457A :
            SetCtrlVal (panel, PANEL_INSTR, "HP 3457A");
            break;
        }
}

int CVICALLBACK Shutdown(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    if (event == EVENT_COMMIT)
        QuitUserInterface(0);
    return 0;
}

