
#include <cvirte.h>    /* Needed if linking in external compiler; harmless otherwise */
#include <userint.h>
#include <stdio.h>
#include "moustate.h"

#ifndef FALSE
#define FALSE 0
#define TRUE !FALSE
#endif


/************************************/
/*  Private Function Protottypes    */
/************************************/

static int ClickIsOnCtrl (int panel, int control, int x, int y);


/********************/
/*      Globals     */
/********************/

static int panelHandle;


/************************/
/*      Functions       */
/************************/

/************************************************************/
/* x and y represent the relative position as calculated by */
/* GetRelativeMouseState, NOT the click location reported   */
/* in left or right click events.  If you pass zero for the */
/* control, reports whether the click is on the panel.      */
/************************************************************/

static int ClickIsOnCtrl (int panel, int control, int x, int y)
{
    int height, width;
    
    /*********************************************************/
    /* If relative position is negative, we are above and/or */
    /* to the left of the panel or ctrl.                     */
    /*********************************************************/
    if ((x < 0) || (y < 0))
        return FALSE;
        
    /*******************************************************/
    /* Get appropriate height and width for panel or ctrl. */
    /*******************************************************/
    if (control == 0) 
    {
        GetPanelAttribute (panel, ATTR_HEIGHT, &height);
        GetPanelAttribute (panel, ATTR_WIDTH, &width);
    }
    else 
    {
        GetCtrlAttribute (panel, control, ATTR_HEIGHT, &height);
        GetCtrlAttribute (panel, control, ATTR_WIDTH, &width);
    }

    /*************************************************************/
    /* If relative positions are less than the width and height, */
    /* the coordinates must be inside the panel or ctrl area.    */
    /*************************************************************/
    if ((y < height) && (x < width))
        return TRUE;
    else
        return FALSE;
}


int CVICALLBACK ShowRelativeHelp (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) 
    {
        case EVENT_COMMIT:
            MessagePopup ("Relative Mouse State", 
                          "The panel coordinates represent the mouse position\n"
                          "relative to the top left corner of the panel.\n\n"
                          "The help button coordinates are relative to the top\n"
                          "left corner of the button.\n\n"
                          "Note that the mouse cursor is changed when the mouse\n"
                          "is over the panel or the help button.");
            break;
    }
    return 0;
}


int CVICALLBACK SetPollingRate (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    double t;
    
    switch (event) 
    {
        case EVENT_COMMIT:
            GetCtrlVal (panel, control, &t);
            SetCtrlAttribute (panel, PANEL_TIMER, ATTR_INTERVAL, t);
            break;
    }
    return 0;
}


int CVICALLBACK ShutDown (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) 
    {
        case EVENT_COMMIT:
            QuitUserInterface (0);
            break;
    }
    return 0;
}


int CVICALLBACK PollTimerCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    int x, y, left, right, keys, panelX, panelY, ctrlX, ctrlY;
    char coords[15];
    
    switch (event) 
    {
        case EVENT_TIMER_TICK:
            /************************************************/
            /* Report global coords and button / key states */
            /************************************************/
            GetGlobalMouseState (NULL, &x, &y, &left, &right, &keys);
            SetCtrlVal (panel, PANEL_LEFT, left);
            SetCtrlVal (panel, PANEL_RIGHT, right);
            sprintf (coords, "(%ld,\n %ld)", x, y);
            SetCtrlVal (panel, PANEL_COORDS, coords);
            SetCtrlVal (panel, PANEL_SHIFT, keys & VAL_SHIFT_MODIFIER);
            SetCtrlVal (panel, PANEL_CTRL, keys & VAL_MENUKEY_MODIFIER);

            /***********************************/
            /* Report coords relative to panel */
            /***********************************/
            GetRelativeMouseState (panel, 0, &panelX, &panelY, NULL,
                                   NULL, NULL);
            sprintf (coords, "(%ld, %ld)", panelX, panelY);
            SetCtrlVal (panel, PANEL_PANELCOORDS, coords);
            
            /*****************************************/
            /* Report coords relative to help button */
            /*****************************************/
            GetRelativeMouseState (panel, PANEL_RELHELP, &ctrlX, &ctrlY, NULL,
                                   NULL, NULL);
            sprintf (coords, "(%ld, %ld)", ctrlX, ctrlY);
            SetCtrlVal (panel, PANEL_CTRLCOORDS, coords);

            /************************************************************/
            /* Change mouse cursor if we are over panel or help button, */
            /* and this panel is the active panel.                      */
            /************************************************************/
            if (GetActivePanel() == panel)
            {
                if (ClickIsOnCtrl (panel, 0, panelX, panelY)) 
                {
                    if (ClickIsOnCtrl (panel, PANEL_RELHELP, ctrlX, ctrlY))
                        SetMouseCursor (VAL_CROSS_HAIR_CURSOR);
                    else            
                        SetMouseCursor (VAL_POINTING_FINGER_CURSOR);
                }
                else
                    SetMouseCursor (VAL_DEFAULT_CURSOR);
            }
            else
                SetMouseCursor (VAL_DEFAULT_CURSOR);
            
            break;
    }
    
#if _NI_sparc_
    ProcessDrawEvents ();
#endif  
    
    return 0;
}


int main (int argc, char *argv[])
{
    if (InitCVIRTE (0, argv, 0) == 0)    /* Needed if linking in external compiler; harmless otherwise */
        return -1;    /* out of memory */
    
    panelHandle = LoadPanel (0, "moustate.uir", PANEL);
    DisplayPanel (panelHandle);
    RunUserInterface ();

    return 0;
}


