/*************************************************************************/
/*                                                                       */
/*  This example illustrates the use of child windows in LabWindows/CVI  */
/*  A child window is a window (panel) that is "owned" by another        */
/*  window (panel).                                                      */
/*                                                                       */
/*************************************************************************/

#include <cvirte.h>    /* Needed if linking in external compiler; harmless otherwise */
#include <userint.h>
#include "panels.h"

int parent_handle, child_1, child_2, grandchild_1, grandchild_2;

int main (int argc, char *argv[])
{
    if (InitCVIRTE (0, argv, 0) == 0)    /* Needed if linking in external compiler; harmless otherwise */
        return -1;    /* out of memory */
    
    parent_handle = LoadPanel (0, "panels.uir", PARENT);
    child_1 = LoadPanel (parent_handle, "panels.uir", CHILD1);
    grandchild_1 = LoadPanel (child_1, "panels.uir", GRNDCHILD1);
    child_2 = LoadPanel (parent_handle, "panels.uir", CHILD2);
    grandchild_2 = LoadPanel (child_2, "panels.uir", GRNDCHILD2);

    DisplayPanel (parent_handle);

    RunUserInterface();

    return 0;
}


int CVICALLBACK ShowHidePanels(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    if (event == EVENT_COMMIT) {
        switch (control) {
            case PARENT_SHOW :
                DisplayPanel (child_1);
                DisplayPanel (grandchild_1);
                DisplayPanel (child_2);
                DisplayPanel (grandchild_2);
                break;
            case PARENT_HIDE :
                HidePanel (child_1);
                HidePanel (child_2);
                break;
            }
        }
    return 0;
}

int CVICALLBACK Shutdown(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    if (event == EVENT_COMMIT)
        QuitUserInterface(0);
    return 0;
}
