/************************************************************************/
/*                                                                      */
/*  This program illustrates how you can use the picture control        */
/*  to simulate animation.  The UIR file has two PCX images; one        */
/*  portraying an open valve and the other a closed valve.  These       */
/*  PCX images have been imported into two picture controls             */
/*  placed on top of each other in the UIR file.  Each time you         */
/*  click on the picture control during runtime, the function           */
/*  ValveChange is called.  ValveChange hides the current valve         */
/*  PCX and displays the other one.  Therefore, by clicking             */
/*  on the PCX, you can simulate an opening and closing valve.          */
/*                                                                      */
/************************************************************************/

#include <cvirte.h>    /* Needed if linking in external compiler; harmless otherwise */
#include <userint.h>
#include "picture.h"


int valve_closed=1;
int handle;

int main (int argc, char *argv[])
{
    if (InitCVIRTE (0, argv, 0) == 0)    /* Needed if linking in external compiler; harmless otherwise */
        return -1;    /* out of memory */
    
    handle = LoadPanel (0, "picture.uir", PANEL);
    SetCtrlAttribute (handle, PANEL_OPEN, ATTR_VISIBLE, 0);
    DisplayPanel (handle);
    RunUserInterface ();

    return 0;
}

int CVICALLBACK TimerCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
double  input, output;

    switch (event) {
        case EVENT_TIMER_TICK:
            if (!valve_closed) {
                GetCtrlVal(handle, PANEL_INPUT, &input);
                if (input > 3.00) {
                    GetCtrlVal(handle, PANEL_OUTPUT, &output);
                    input -= 0.01;
                    output += 0.01;
                    SetCtrlVal(handle, PANEL_INPUT, input);
                    SetCtrlVal(handle, PANEL_OUTPUT, output);
                    }
                }
            break;
    }
    
    return 0;
}


int CVICALLBACK ValveChange(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    if ((event == EVENT_LEFT_CLICK) || (event == EVENT_LEFT_DOUBLE_CLICK)) {
        if (valve_closed) {
            SetCtrlAttribute (handle, PANEL_OPEN, ATTR_VISIBLE, 1);
            SetCtrlAttribute (handle, PANEL_CLOSED, ATTR_VISIBLE, 0);
            valve_closed = 0;
            }
        else {
            SetCtrlAttribute (handle, PANEL_OPEN, ATTR_VISIBLE, 0);
            SetCtrlAttribute (handle, PANEL_CLOSED, ATTR_VISIBLE, 1);
            valve_closed = 1;
            }
        }
        
    return 0;
}

int CVICALLBACK Shutdown(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    if (event == EVENT_COMMIT)
        QuitUserInterface(0);
        
    return 0;
}

int CVICALLBACK Reset(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    if (event == EVENT_COMMIT)
        {
        SetCtrlVal(handle, PANEL_INPUT, 7.00);
        SetCtrlVal(handle, PANEL_OUTPUT, 3.00);
        }
        
    return 0;
}
