#include <formatio.h>
#include <ansi_c.h>
#include <userint.h>
#include "piechart.h"

void GetArcPoints(Point *arcPoints);

/* Circle attributes */
static Point origin, *arcPoints;
static int radius;
static double angleRad, lastAngleRad;
static int canvasBGColor;  
static int labelWidth;       

int CreatePie(int pnlHandle, pieInfo pieAttr, piePiece piece, int legend)
{
    static int pieID, i, memoryForPoints, j;
    int sumOfPieces=0, otherPiece=0;
    char otherLegend[50];
    
    /* Initialize variables */
    angleRad = 0;
    lastAngleRad = 0;
    
    /* Sum percentages */
    for (i=0;i<piece.numPieces;i++)
        sumOfPieces += piece.piecePercent[i];
    
    /* Return error -10000 if sum of percentages is > 100 */
    if (sumOfPieces>100)
        return -10000;
    else if (sumOfPieces<100)
        otherPiece = 100 - sumOfPieces;  /* Create otherPiece to be difference */
    
    /* Create canvas control */
    pieID = NewCtrl (pnlHandle, CTRL_CANVAS, pieAttr.title, pieAttr.top,
                        pieAttr.left);
    if (pieID<0)
        return pieID;
        
    /* Size canvas control depending whether legend is requested */
    SetCtrlAttribute (pnlHandle, pieID, ATTR_HEIGHT, pieAttr.height);
    if (legend)
        SetCtrlAttribute (pnlHandle, pieID, ATTR_WIDTH, pieAttr.width+LEGENDWIDTH);
    else
        SetCtrlAttribute (pnlHandle, pieID, ATTR_WIDTH, pieAttr.width);
        
    /* Position label */
    GetCtrlAttribute (pnlHandle, pieID, ATTR_LABEL_WIDTH, &labelWidth);
    SetCtrlAttribute (pnlHandle, pieID, ATTR_LABEL_LEFT,
                    pieAttr.left+(pieAttr.width/2)-(labelWidth/2));
    
    /* Determine origin of pie */
    if (pieAttr.height>pieAttr.width) {
        origin.x = pieAttr.width/2;
        origin.y = pieAttr.width/2;
        }
    else {
        origin.x = pieAttr.height/2;
        origin.y = pieAttr.height/2;
        }
        
    radius = origin.x;
    
    /* Start canvas batch drawing */
    CanvasStartBatchDraw (pnlHandle, pieID);
    
    j = -2;
    
    /* For all pieces with percentages > 0, create arc (using GetArcPoints) */
    /* and draw arc using CanvasDrawPoly().  Create legend if legend was    */
    /* requested.                                                           */
    for (i=0;i<piece.numPieces;i++) {
        if (piece.piecePercent[i] > 0) {
            j++;
            angleRad += piece.piecePercent[i]*2*PI/100;
            memoryForPoints = sizeof(Point)*((angleRad-lastAngleRad)/(radius*DETAILLEVEL)+3);
            arcPoints = malloc(memoryForPoints);
            GetArcPoints(arcPoints);
            SetCtrlAttribute (pnlHandle, pieID, ATTR_PEN_FILL_COLOR, piece.colors[i]);
            CanvasDrawPoly (pnlHandle, pieID, memoryForPoints/sizeof(Point),
                            arcPoints, 1, VAL_DRAW_FRAME_AND_INTERIOR);
            if (legend) {
                CanvasDrawRect (pnlHandle, pieID, 
                            MakeRect (pieAttr.top+(20*j), pieAttr.width+20, 15, 15),
                            VAL_DRAW_FRAME_AND_INTERIOR);
                SetCtrlAttribute (pnlHandle, pieID, ATTR_PEN_FILL_COLOR,
                                  VAL_TRANSPARENT);
                Fmt(piece.legendInfo[i], "%s[a]< (%d%%)", piece.piecePercent[i]);
                CanvasDrawTextAtPoint (pnlHandle, pieID, piece.legendInfo[i],
                                   VAL_APP_META_FONT, 
                                   MakePoint (pieAttr.width+40, pieAttr.top+(20*j)),
                                   VAL_UPPER_LEFT);
                }
            free(arcPoints);
            }
        }
        
    /* Draw arc for last "other" wedge, if it exists */
    if (otherPiece>0) {
        j++;
        angleRad = 2*PI;
        memoryForPoints = sizeof(Point)*((angleRad-lastAngleRad)/(radius*DETAILLEVEL)+3);
        arcPoints = malloc(memoryForPoints);
        GetArcPoints(arcPoints);
        SetCtrlAttribute (pnlHandle, pieID, ATTR_PEN_FILL_COLOR, VAL_DK_GRAY);
        CanvasDrawPoly (pnlHandle, pieID, memoryForPoints/sizeof(Point),
                        arcPoints, 1, VAL_DRAW_FRAME_AND_INTERIOR);
        if (legend) {
            CanvasDrawRect (pnlHandle, pieID, 
                        MakeRect (pieAttr.top+(20*j), pieAttr.width+20, 15, 15),
                        VAL_DRAW_FRAME_AND_INTERIOR);
            SetCtrlAttribute (pnlHandle, pieID, ATTR_PEN_FILL_COLOR,
                                  VAL_TRANSPARENT);
            Fmt(otherLegend, "%s<Other (%d%%)", otherPiece);
            CanvasDrawTextAtPoint (pnlHandle, pieID, otherLegend,
                               VAL_APP_META_FONT, 
                               MakePoint (pieAttr.width+40, pieAttr.top+(20*j)),
                               VAL_UPPER_LEFT);
            }
        free(arcPoints);
        }
    
    CanvasEndBatchDraw (pnlHandle, pieID);

    return pieID;
}


/* This routine creates points for an arc.  */

void GetArcPoints(Point *arcPoints)
{
    static double i;
    static double x, y;
    static int j;
    j=0;
    
    arcPoints[0] = MakePoint (radius, radius);
    
    for (i=lastAngleRad;i<angleRad;i+=radius*DETAILLEVEL) {
        x = radius*cos(i);
        y = radius*sin(i);
        arcPoints[++j] = MakePoint (radius + x, radius - y);
        }
    x = radius*cos(angleRad);
    y = radius*sin(angleRad);
    arcPoints[++j] = MakePoint (radius + x, radius - y);
    
    lastAngleRad = angleRad;
}
