/* Pie Chart on Canvas Control Sample Program */

#define HELPTEXT "\
This sample program uses a simple Pie Chart instrument driver\n\
that adds a Canvas Control to an existing panel and draws a pie\n\
chart on the canvas control."

#include <cvirte.h>     /* needed if linking executable in external compiler; harmless otherwise */
#include <formatio.h>
#include <ansi_c.h>
#include <userint.h>
#include "piedemo.h"
#include "piechart.h"

static int panelHandle;

static int secondPanel, firstPanel;
pieInfo pieAttributes;
piePiece pieChart;
int pieChartID;
static int i;

int main (int argc, char *argv[])
{
    if (InitCVIRTE (0, argv, 0) == 0)    /* Needed if linking in external compiler; harmless otherwise */
        return -1;    /* out of memory */
    
    /* Initialize Pie Attribute structure */
    pieAttributes.title = malloc(100);
    pieAttributes.top = 30;
    pieAttributes.left = 20;
    pieAttributes.width = 300;
    pieAttributes.height = 325;
    Fmt(pieAttributes.title, "SPC Analysis of Production Line Failures");
    
    /* Initialize Pie Chart structure */
    pieChart.numPieces = 6;
    pieChart.legendInfo = malloc(pieChart.numPieces*sizeof(int));
    for (i=0;i<pieChart.numPieces;i++)
        pieChart.legendInfo[i] = malloc(50);
        
    pieChart.piecePercent = malloc(sizeof(int)*pieChart.numPieces);
    pieChart.colors = malloc(sizeof(int)*pieChart.numPieces);
    
    /* Initialize Pie Chart legend info */  
    Fmt(pieChart.legendInfo[0], "Bent Lead");
    Fmt(pieChart.legendInfo[1], "Loose Wire");
    Fmt(pieChart.legendInfo[2], "Bad Part");
    Fmt(pieChart.legendInfo[3], "Component Failure");
    Fmt(pieChart.legendInfo[4], "Bad Power Module");
    Fmt(pieChart.legendInfo[5], "Operator Error");
    
    /* Initialize Pie Chart Data */
    pieChart.piecePercent[0] = 38;
    pieChart.piecePercent[1] = 27;
    pieChart.piecePercent[2] = 11;
    pieChart.piecePercent[3] = 11;
    pieChart.piecePercent[4] = 8;
    pieChart.piecePercent[5] = 5;
    pieChart.colors[0] = VAL_RED;
    pieChart.colors[1] = VAL_BLUE;
    pieChart.colors[2] = VAL_GREEN;
    pieChart.colors[3] = VAL_MAGENTA;
    pieChart.colors[4] = VAL_YELLOW;
    pieChart.colors[5] = VAL_DK_BLUE;
    
    /* Draw Pie Chart on Canvas Control using CreatePie Function */    
    if ((firstPanel = LoadPanel (0, "piedemo.uir", PANEL)) < 0)
        return -1;
    pieChartID = CreatePie (firstPanel, pieAttributes, pieChart, VAL_LEGENDON); 
    SetCtrlAttribute (firstPanel, pieChartID, ATTR_ZPLANE_POSITION, 1000);
    DisplayPanel(firstPanel);
    
    RunUserInterface ();

    return 0;
}

int CVICALLBACK Quit (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            free(pieAttributes.title);
            for (i=0;i<pieChart.numPieces;i++) {
                free(pieChart.legendInfo[i]);
                }
            free(pieChart.legendInfo);
            free(pieChart.piecePercent);
            free(pieChart.colors);
            QuitUserInterface (0);
            break;
    }
    return 0;
}

int CVICALLBACK Help (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            MessagePopup ("Pie Chart on Canvas Control", HELPTEXT);
            break;
    }
    return 0;
}
