/* Arbitrary Labels(Time) for a Graph Axis */

#define HELPTEXT "\
This sample program illustrates the use of arbitrary axis labels for either\n\
graphs or stripcharts.  Notice that X-axis labels are labeled as time increments\n\
instead of integer values. Moving the graph cursor on the graph with the mouse will\n\
update the time value displayed in the Graph Details area of the panel.\n\n\
The two functions used to set and clear the labels for the axis are:\n\
     InsertAxisItem and\n\
     ClearAxisItems."

#include <cvirte.h>     /* needed if linking executable in external compiler; harmless otherwise */
#include <userint.h>
#include <stdlib.h>
#include <ansi_c.h>
#include "timeaxis.h"

// Format24
#define AM_PM   1
#define HOUR24  0

// FormatHMS
#define HHMM    1
#define HHMMSS  0

static int Str2Sec(char *Str, int *Second);
static int Sec2Str(int Second, char *Str, int Format24, int FormatHMS);
static int SetXHoursLabel(int panel, int control, int StartValue, int DeltaValue, char *Start, char *Delta, char *Total, int Format24, int FormatHMS);
static int GetXHoursCursor(int panel, int control, int cursor, char *Str, int Format24, int FormatHMS);

static int panelHandle;

int main (int argc, char *argv[])
{
    if (InitCVIRTE (0, argv, 0) == 0)    /* Needed if linking in external compiler; harmless otherwise */
        return -1;    /* out of memory */
        
    /* Load Panel */
    if ((panelHandle = LoadPanel (0, "timeaxis.uir", PANEL)) < 0)
        return -1;
    /* Update graph */
    Generate(panelHandle, NULL, EVENT_COMMIT, NULL, 0, 0);
    /* Force Graph cusor output control to update */
    GraphCallback (panelHandle, PANEL_GRAPH, EVENT_VAL_CHANGED, 0, 0, 0);
    
    DisplayPanel (panelHandle);
    RunUserInterface ();

    return 0;
}

int CVICALLBACK GraphCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2) {
    char    Str[20];
    int     Format24, FormatHMS;      
    
    switch (event) {
        case EVENT_VAL_CHANGED:
            GetCtrlVal(panel, PANEL_FORMAT24, &Format24);         
            GetCtrlVal(panel, PANEL_FORMAT_HMS, &FormatHMS);
            GetXHoursCursor(panel, control, 1, Str, Format24, FormatHMS); 
            SetCtrlVal(panel, PANEL_HOUR, Str);
        break;
    }
    return 0;
}

int CVICALLBACK RedrawLabels(int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2) {
    
    int Format24, FormatHMS, SaveCursor;
    char StartTime[9], DeltaTime[9], TotalTime[9];
    
    switch (event) {
        case EVENT_COMMIT:
            GetCtrlVal(panel, PANEL_FORMAT24, &Format24);         
            GetCtrlVal(panel, PANEL_FORMAT_HMS, &FormatHMS);
            GetCtrlVal(panel, PANEL_STARTTIME, StartTime);
            
            GetCtrlVal(panel, PANEL_DELTATIME, DeltaTime);
            GetCtrlVal(panel, PANEL_TOTALTIME, TotalTime);
            GetMouseCursor (&SaveCursor);
            SetMouseCursor (VAL_HOUR_GLASS_CURSOR);
            SetXHoursLabel(panelHandle, PANEL_GRAPH, 0, 1, StartTime, DeltaTime, TotalTime, Format24, FormatHMS);
            SetMouseCursor (SaveCursor);
            
        break;
    }
    return 0;
}

int CVICALLBACK Generate (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2) {
    
    static double Array[1024];
    int Pts, i;
    
    switch (event) {
        case EVENT_COMMIT:
            GetCtrlVal(panel, PANEL_NB_PTS, &Pts);
            for (i=0; i<Pts; i++){
                Array[i]=100.0 * rand()/RAND_MAX;
            }
            DeleteGraphPlot (panel, PANEL_GRAPH, -1, VAL_DELAYED_DRAW);
            PlotY (panel, PANEL_GRAPH, Array, Pts, VAL_DOUBLE, VAL_THIN_LINE,
                         VAL_EMPTY_SQUARE, VAL_SOLID, 1, VAL_DK_BLUE);
            RedrawLabels(panel, NULL, EVENT_COMMIT, NULL, 0, 0);
            SetActiveCtrl(panel, PANEL_GRAPH);
        break;
    }
    return 0;
}

int CVICALLBACK Quit (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2) {
    switch (event) {
        case EVENT_COMMIT:
            QuitUserInterface (0);
        break;
    }
    return 0;
}

int CVICALLBACK Help (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_COMMIT:
            MessagePopup ("Graph with Time Axis Sample Program", HELPTEXT);
            break;
    }
    return 0;
}

/* Set the labels of a graph to specify time */
static int SetXHoursLabel(int panel, int control, int StartValue, int DeltaValue, char *Start, char *Delta, char *Total, int Format24, int FormatHMS) {
    int     StartSecond, DeltaSecond, TotalSecond, TempSecond, Dum;
    char    Str[32];
    double  i, Min, Max;
    int     NumPts, LabelArrayIndex;
    double  Step;
    
    if (Start!=NULL)
        Str2Sec(Start, &StartSecond);
    else StartSecond=0;
    
    if(Delta!=NULL)    
        Str2Sec(Delta, &DeltaSecond);
    else DeltaSecond=1;
    
    if(Total!=NULL)    
        Str2Sec(Total, &TotalSecond);
    else TotalSecond=1;
    
    GetCtrlVal (panel, PANEL_NB_PTS, &NumPts);
    SetCtrlAttribute (panel, control, ATTR_XUSE_LABEL_STRINGS, 1);
    ClearAxisItems (panel, control, VAL_XAXIS);
    TempSecond = StartValue + TotalSecond + StartSecond;
    
    
    for (i=StartValue+StartSecond; i<TempSecond; i=i+DeltaSecond){
        Sec2Str(i, Str, Format24, FormatHMS);
        sprintf(Str, "%s ", Str);
        LabelArrayIndex = (int)((i - StartSecond)*NumPts/TotalSecond);
        InsertAxisItem (panel, control, VAL_XAXIS, -1, Str, LabelArrayIndex);
    }
    
    if (i == TempSecond)
    {
        Sec2Str(i, Str, Format24, FormatHMS);
        sprintf(Str, "%s ", Str);
        LabelArrayIndex = NumPts - 1;
        InsertAxisItem (panel, control, VAL_XAXIS, -1, Str, LabelArrayIndex);
    }
    GetXHoursCursor (panel, control, 1, Str, Format24, FormatHMS);
    SetCtrlVal (panel, PANEL_HOUR, Str);
    return(0);
}

/* Convert the cursor numberic value into a Time String */
static int GetXHoursCursor(int panel, int control, int cursor, char *Str, int Format24, int FormatHMS){
    
    double X, Y;            
    int Plot, Index, NumPts;
    char TotalTime[9], StartTime[9];
    int TotalSec, StartSec;

    GetGraphCursor (panel, control, 1, &X, &Y);
    GetCtrlVal (panel, PANEL_NB_PTS, &NumPts);
    GetCtrlVal (panel, PANEL_STARTTIME, StartTime);
    GetCtrlVal (panel, PANEL_TOTALTIME, TotalTime);
    Str2Sec(StartTime, &StartSec);
    Str2Sec(TotalTime, &TotalSec);
    
    X = X / NumPts * TotalSec + StartSec;
    
    Sec2Str(X, Str, Format24, FormatHMS);
    return(0);
}

/* Convert a specified number of Seconds into a Time string */
static int Sec2Str(int Second, char *Str, int Format24, int FormatHMS){
    
    int Sec, Min, Hour;
    int CpySecond=Second;
    
    Hour = Second/3600; 
    Second = Second - (3600*Hour);   
    if(Format24==AM_PM)
        Hour%=12;
    else
        Hour%=24;
    
    Min = Second/60;  
    Second = Second - (60*Min);
    Sec = Second;
    switch(FormatHMS){
        case HHMM:
            sprintf(Str, "%02d:%02d", Hour, Min);     
        break;
        case HHMMSS:
            sprintf(Str, "%02d:%02d:%02d", Hour, Min, Sec);
        break;  
    }
    Second=CpySecond;
    if (Format24==AM_PM){
        if (Second>=43200)          //43200=12*3600
            strcat(Str, " PM");
        else
            strcat(Str, " AM");
    }
    return(0);
}

/* Convert a Time string into Seconds */
static int Str2Sec(char *Str, int *Second){
    int Sec, Min, Hour;
    char *bob;
    int i=3600;
    
    *Second=0;
    bob = strtok (Str, ":");
    while(bob){
        *Second = *Second+ i*atoi(bob);
        i=i/60;      
        bob = strtok (NULL, ":");
    }
    return(0);
}
