/************************************************************************/
/*                                                                      */
/*  This program shows how you can use a timer control to               */
/*  respond to user interface events without locking your user          */
/*  interface.  For example, you can operate controls on a pop-up       */
/*  panel without halting a strip chart display on the panel below the  */
/*  popup.                                                              */
/*                                                                      */
/************************************************************************/

#include <cvirte.h>    /* Needed if linking in external compiler; harmless otherwise */
#include <userint.h>
#include <ansi_c.h>
#include "timerctl.h"

int goflag=0;
int handle, popuphandle;
double datapoints[100];

int main (int argc, char *argv[])
{
    if (InitCVIRTE (0, argv, 0) == 0)    /* Needed if linking in external compiler; harmless otherwise */
        return -1;    /* out of memory */
    
    handle = LoadPanel (0, "timerctl.uir", PANEL);
    popuphandle = LoadPanel (0, "timerctl.uir", POPUP);
    DisplayPanel (handle);
    RunUserInterface ();

    return 0;
}

int CVICALLBACK TimerCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_TIMER_TICK:
            if (goflag == 1) {
                datapoints[0] = 100.0 * rand() / RAND_MAX;
                PlotStripChart (handle, PANEL_CHART, datapoints, 1, 0, 0, VAL_DOUBLE);
                }
            break;
        }
    
    return 0;
}

int CVICALLBACK GoFunction(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    if (event == EVENT_COMMIT)
        goflag = 1;
        
    return 0;
}

int CVICALLBACK PopupInfo(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    int pnl=0, ctrl=0, quit=0;

        if (event == EVENT_COMMIT) {
            InstallPopup (popuphandle);
            while (!quit) {
                GetUserEvent (1, &pnl, &ctrl);
                if (ctrl == POPUP_OK) {
                    HidePanel (popuphandle);
                    quit=1;
                    }
                }
            }
            
    return 0;
}

int CVICALLBACK Shutdown(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    if (event == EVENT_COMMIT)
        QuitUserInterface(0);
        
    return 0;
}
